#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
#
# Copyright (C) 2017 Lenovo, Inc.
# (c) 2017, Ansible by Red Hat, inc
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
# Module to work on Link Aggregation with Lenovo Switches
# Lenovo Networking
#
DOCUMENTATION = '''
---
module: cnos_linkagg
author: "Anil Kumar Muraleedharan (@auraleedhar)"
short_description: Manage link aggregation groups on Lenovo CNOS devices
description:
  - This module provides declarative management of link aggregation groups
    on Lenovo CNOS network devices.
notes:
  - Tested against CNOS 10.8.1
options:
  group:
    description:
      - Channel-group number for the port-channel
        Link aggregation group. Range 1-255.
  mode:
    description:
      - Mode of the link aggregation group.
    choices: ['active', 'on', 'passive']
  members:
    description:
      - List of members of the link aggregation group.
  aggregate:
    description: List of link aggregation definitions.
  state:
    description:
      - State of the link aggregation group.
    default: present
    choices: ['present', 'absent']
  purge:
    description:
      - Purge links not defined in the I(aggregate) parameter.
    type: bool
    default: no
  provider:
    description:
      - B(Deprecated)
      - "Starting with Ansible 2.5 we recommend using C(connection: network_cli)."
      - For more information please see the L(CNOS Platform Options guide, ../network/user_guide/platform_cnos.html).
      - HORIZONTALLINE
      - A dict object containing connection details.
    suboptions:
      host:
        description:
          - Specifies the DNS host name or address for connecting to the remote
            device over the specified transport.  The value of host is used as
            the destination address for the transport.
        required: true
      port:
        description:
          - Specifies the port to use when building the connection to the remote device.
        default: 22
      username:
        description:
          - Configures the username to use to authenticate the connection to
            the remote device.  This value is used to authenticate
            the SSH session. If the value is not specified in the task, the
            value of environment variable C(ANSIBLE_NET_USERNAME) will be used instead.
      password:
        description:
          - Specifies the password to use to authenticate the connection to
            the remote device.   This value is used to authenticate
            the SSH session. If the value is not specified in the task, the
            value of environment variable C(ANSIBLE_NET_PASSWORD) will be used instead.
      timeout:
        description:
          - Specifies the timeout in seconds for communicating with the network device
            for either connecting or sending commands.  If the timeout is
            exceeded before the operation is completed, the module will error.
        default: 10
      ssh_keyfile:
        description:
          - Specifies the SSH key to use to authenticate the connection to
            the remote device.   This value is the path to the
            key used to authenticate the SSH session. If the value is not specified
            in the task, the value of environment variable C(ANSIBLE_NET_SSH_KEYFILE)
            will be used instead.
      authorize:
        description:
          - Instructs the module to enter privileged mode on the remote device
            before sending any commands.  If not specified, the device will
            attempt to execute all commands in non-privileged mode. If the value
            is not specified in the task, the value of environment variable
            C(ANSIBLE_NET_AUTHORIZE) will be used instead.
        type: bool
        default: 'no'
      auth_pass:
        description:
          - Specifies the password to use if required to enter privileged mode
            on the remote device.  If I(authorize) is false, then this argument
            does nothing. If the value is not specified in the task, the value of
            environment variable C(ANSIBLE_NET_AUTH_PASS) will be used instead.
'''

EXAMPLES = """
- name: Create link aggregation group
  community.network.cnos_linkagg:
    group: 10
    state: present

- name: Delete link aggregation group
  community.network.cnos_linkagg:
    group: 10
    state: absent

- name: Set link aggregation group to members
  community.network.cnos_linkagg:
    group: 200
    mode: active
    members:
      - Ethernet1/33
      - Ethernet1/44

- name: Remove link aggregation group from GigabitEthernet0/0
  community.network.cnos_linkagg:
    group: 200
    mode: active
    members:
      - Ethernet1/33

- name: Create aggregate of linkagg definitions
  community.network.cnos_linkagg:
    aggregate:
      - { group: 3, mode: on, members: [Ethernet1/33] }
      - { group: 100, mode: passive, members: [Ethernet1/44] }
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always, except for the platforms that use Netconf transport to
    manage the device.
  type: list
  sample:
    - interface port-channel 30
    - interface Ethernet1/33
    - channel-group 30 mode on
    - no interface port-channel 30
"""

import re
from copy import deepcopy

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import CustomNetworkConfig
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import remove_default_spec
from ansible_collections.community.network.plugins.module_utils.network.cnos.cnos import get_config, load_config
from ansible_collections.community.network.plugins.module_utils.network.cnos.cnos import cnos_argument_spec


def search_obj_in_list(group, lst):
    for o in lst:
        if o['group'] == group:
            return o


def map_obj_to_commands(updates, module):
    commands = list()
    want, have = updates
    purge = module.params['purge']

    for w in want:
        group = w['group']
        mode = w['mode']
        members = w.get('members') or []
        state = w['state']
        del w['state']

        obj_in_have = search_obj_in_list(group, have)

        if state == 'absent':
            if obj_in_have:
                commands.append('no interface port-channel {0}'.format(group))

        elif state == 'present':
            cmd = ['interface port-channel {0}'.format(group),
                   'exit']
            if not obj_in_have:
                if not group:
                    module.fail_json(msg='group is a required option')
                commands.extend(cmd)

                if members:
                    for m in members:
                        commands.append('interface {0}'.format(m))
                        commands.append('channel-group {0} mode {1}'.format(group, mode))

            else:
                if members:
                    if 'members' not in obj_in_have.keys():
                        for m in members:
                            commands.extend(cmd)
                            commands.append('interface {0}'.format(m))
                            commands.append('channel-group {0} mode {1}'.format(group, mode))

                    elif set(members) != set(obj_in_have['members']):
                        missing_members = list(set(members) - set(obj_in_have['members']))
                        for m in missing_members:
                            commands.extend(cmd)
                            commands.append('interface {0}'.format(m))
                            commands.append('channel-group {0} mode {1}'.format(group, mode))

                        superfluous_members = list(set(obj_in_have['members']) - set(members))
                        for m in superfluous_members:
                            commands.extend(cmd)
                            commands.append('interface {0}'.format(m))
                            commands.append('no channel-group')

    if purge:
        for h in have:
            obj_in_want = search_obj_in_list(h['group'], want)
            if not obj_in_want:
                commands.append('no interface port-channel {0}'.format(h['group']))

    return commands


def map_params_to_obj(module):
    obj = []

    aggregate = module.params.get('aggregate')
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]

            d = item.copy()
            d['group'] = str(d['group'])

            obj.append(d)
    else:
        obj.append({
            'group': str(module.params['group']),
            'mode': module.params['mode'],
            'members': module.params['members'],
            'state': module.params['state']
        })

    return obj


def parse_mode(module, config, group, member):
    mode = None
    netcfg = CustomNetworkConfig(indent=1, contents=config)
    parents = ['interface {0}'.format(member)]
    body = netcfg.get_section(parents)

    match_int = re.findall(r'interface {0}\n'.format(member), body, re.M)
    if match_int:
        match = re.search(r'channel-group {0} mode (\S+)'.format(group),
                          body, re.M)
        if match:
            mode = match.group(1)

    return mode


def parse_members(module, config, group):
    members = []

    for line in config.strip().split('!'):
        l = line.strip()
        if l.startswith('interface'):
            match_group = re.findall(r'channel-group {0} mode'.format(group), l, re.M)
            if match_group:
                match = re.search(r'interface (\S+)', l, re.M)
                if match:
                    members.append(match.group(1))

    return members


def get_channel(module, config, group):
    match = re.findall(r'^interface (\S+)', config, re.M)

    if not match:
        return {}

    channel = {}
    for item in set(match):
        member = item
        channel['mode'] = parse_mode(module, config, group, member)
        channel['members'] = parse_members(module, config, group)

    return channel


def map_config_to_obj(module):
    objs = list()
    config = get_config(module)

    for line in config.split('\n'):
        l = line.strip()
        match = re.search(r'interface port-channel(\S+)', l, re.M)
        if match:
            obj = {}
            group = match.group(1)
            obj['group'] = group
            obj.update(get_channel(module, config, group))
            objs.append(obj)

    return objs


def main():
    """ main entry point for module execution
    """
    element_spec = dict(
        group=dict(type='int'),
        mode=dict(choices=['active', 'on', 'passive']),
        members=dict(type='list'),
        state=dict(default='present',
                   choices=['present', 'absent'])
    )

    aggregate_spec = deepcopy(element_spec)
    aggregate_spec['group'] = dict(required=True)

    required_one_of = [['group', 'aggregate']]
    required_together = [['members', 'mode']]
    mutually_exclusive = [['group', 'aggregate']]

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    argument_spec = dict(
        aggregate=dict(type='list', elements='dict', options=aggregate_spec,
                       required_together=required_together),
        purge=dict(default=False, type='bool')
    )

    argument_spec.update(element_spec)
    argument_spec.update(cnos_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec,
                           required_one_of=required_one_of,
                           required_together=required_together,
                           mutually_exclusive=mutually_exclusive,
                           supports_check_mode=True)

    warnings = list()
    result = {'changed': False}
    if warnings:
        result['warnings'] = warnings

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands((want, have), module)
    result['commands'] = commands

    if commands:
        if not module.check_mode:
            load_config(module, commands)
        result['changed'] = True

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: nxos_bgp_neighbor
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: (deprecated, removed after 2023-01-27) Manages BGP neighbors configurations.
description:
- Manages BGP neighbors configurations on NX-OS switches.
version_added: 1.0.0
author: Gabriele Gerbino (@GGabriele)
deprecated:
  alternative: nxos_bgp_global
  why: Updated module released with more functionality.
  removed_at_date: '2023-01-27'
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- C(state=absent) removes the whole BGP neighbor configuration.
- Default, where supported, restores params default value.
options:
  asn:
    description:
    - BGP autonomous system number. Valid values are string, Integer in ASPLAIN or
      ASDOT notation.
    required: true
    type: str
  vrf:
    description:
    - Name of the VRF. The name 'default' is a valid VRF representing the global bgp.
    default: default
    type: str
  neighbor:
    description:
    - Neighbor Identifier. Valid values are string. Neighbors may use IPv4 or IPv6
      notation, with or without prefix length.
    required: true
    type: str
  description:
    description:
    - Description of the neighbor.
    type: str
  bfd:
    description:
    - Enables/Disables BFD for a given neighbor.
    - "Dependency: ''feature bfd''"
    type: str
    choices:
    - enable
    - disable
  connected_check:
    description:
    - Configure whether or not to check for directly connected peer.
    type: bool
  capability_negotiation:
    description:
    - Configure whether or not to negotiate capability with this neighbor.
    type: bool
  dynamic_capability:
    description:
    - Configure whether or not to enable dynamic capability.
    type: bool
  ebgp_multihop:
    description:
    - Specify multihop TTL for a remote peer. Valid values are integers between 2
      and 255, or keyword 'default' to disable this property.
    type: str
  local_as:
    description:
    - Specify the local-as number for the eBGP neighbor. Valid values are String or
      Integer in ASPLAIN or ASDOT notation, or 'default', which means not to configure
      it.
    type: str
  log_neighbor_changes:
    description:
    - Specify whether or not to enable log messages for neighbor up/down event.
    choices:
    - enable
    - disable
    - inherit
    type: str
  low_memory_exempt:
    description:
    - Specify whether or not to shut down this neighbor under memory pressure.
    type: bool
  maximum_peers:
    description:
    - Specify Maximum number of peers for this neighbor prefix Valid values are between
      1 and 1000, or 'default', which does not impose the limit. Note that this parameter
      is accepted only on neighbors with address/prefix.
    type: str
  pwd:
    description:
    - Specify the password for neighbor. Valid value is string.
    type: str
  pwd_type:
    description:
    - Specify the encryption type the password will use. Valid values are '3des' or
      'cisco_type_7' encryption or keyword 'default'.
    choices:
    - 3des
    - cisco_type_7
    - default
    type: str
  remote_as:
    description:
    - Specify Autonomous System Number of the neighbor. Valid values are String or
      Integer in ASPLAIN or ASDOT notation, or 'default', which means not to configure
      it.
    type: str
  remove_private_as:
    description:
    - Specify the config to remove private AS number from outbound updates. Valid
      values are 'enable' to enable this config, 'disable' to disable this config,
      'all' to remove all private AS number, or 'replace-as', to replace the private
      AS number.
    choices:
    - enable
    - disable
    - all
    - replace-as
    type: str
  shutdown:
    description:
    - Configure to administratively shutdown this neighbor.
    type: bool
  suppress_4_byte_as:
    description:
    - Configure to suppress 4-byte AS Capability.
    type: bool
  timers_keepalive:
    description:
    - Specify keepalive timer value. Valid values are integers between 0 and 3600
      in terms of seconds, or 'default', which is 60.
    type: str
  timers_holdtime:
    description:
    - Specify holdtime timer value. Valid values are integers between 0 and 3600 in
      terms of seconds, or 'default', which is 180.
    type: str
  transport_passive_only:
    description:
    - Specify whether or not to only allow passive connection setup. Valid values
      are 'true', 'false', and 'default', which defaults to 'false'. This property
      can only be configured when the neighbor is in 'ip' address format without prefix
      length.
    type: bool
  update_source:
    description:
    - Specify source interface of BGP session and updates.
    type: str
  state:
    description:
    - Determines whether the config should be present or not on the device.
    default: present
    choices:
    - present
    - absent
    type: str
  peer_type:
    description:
    - Specify the peer type for BGP session.
    choices:
    - fabric_border_leaf
    - fabric_external
    - disable
    type: str
    version_added: 1.1.0
"""
EXAMPLES = """
# create a new neighbor
- cisco.nxos.nxos_bgp_neighbor:
    asn: 65535
    neighbor: 192.0.2.3
    local_as: 20
    remote_as: 30
    bfd: enable
    description: just a description
    update_source: Ethernet1/3
    state: present
    peer_type: fabric_external
"""

RETURN = """
commands:
  description: commands sent to the device
  returned: always
  type: list
  sample: ["router bgp 65535", "neighbor 192.0.2.3",
           "remote-as 30", "update-source Ethernet1/3",
           "description just a description", "local-as 20", "peer-type fabric-external"]
"""

import re

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_config,
    load_config,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    nxos_argument_spec,
)
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    CustomNetworkConfig,
)


BOOL_PARAMS = [
    "capability_negotiation",
    "shutdown",
    "connected_check",
    "dynamic_capability",
    "low_memory_exempt",
    "suppress_4_byte_as",
    "transport_passive_only",
]
PARAM_TO_COMMAND_KEYMAP = {
    "asn": "router bgp",
    "bfd": "bfd",
    "capability_negotiation": "dont-capability-negotiate",
    "connected_check": "disable-connected-check",
    "description": "description",
    "dynamic_capability": "dynamic-capability",
    "ebgp_multihop": "ebgp-multihop",
    "local_as": "local-as",
    "log_neighbor_changes": "log-neighbor-changes",
    "low_memory_exempt": "low-memory exempt",
    "maximum_peers": "maximum-peers",
    "neighbor": "neighbor",
    "pwd": "password",
    "pwd_type": "password",
    "remote_as": "remote-as",
    "remove_private_as": "remove-private-as",
    "shutdown": "shutdown",
    "suppress_4_byte_as": "capability suppress 4-byte-as",
    "timers_keepalive": "timers",
    "timers_holdtime": "timers",
    "transport_passive_only": "transport connection-mode passive",
    "update_source": "update-source",
    "vrf": "vrf",
    "peer_type": "peer-type",
}
PARAM_TO_DEFAULT_KEYMAP = {
    "bfd": "disable",
    "shutdown": False,
    "dynamic_capability": True,
    "timers_keepalive": 60,
    "timers_holdtime": 180,
    "peer_type": "disable",
}


def get_value(arg, config):
    command = PARAM_TO_COMMAND_KEYMAP[arg]
    has_command = re.search(r"^\s+{0}$".format(command), config, re.M)
    has_command_val = re.search(
        r"(?:\s+{0}\s*)(?P<value>.*)$".format(command), config, re.M
    )

    if arg == "dynamic_capability":
        has_no_command = re.search(
            r"\s+no\s{0}\s*$".format(command), config, re.M
        )
        value = True
        if has_no_command:
            value = False
    elif arg in BOOL_PARAMS:
        value = False
        if has_command:
            value = True
    elif arg == "log_neighbor_changes":
        value = ""
        if has_command:
            value = "enable"
        elif has_command_val:
            value = "disable"

    elif arg == "remove_private_as":
        value = "disable"
        if has_command:
            value = "enable"
        elif has_command_val:
            value = has_command_val.group("value")
    elif arg == "bfd":
        value = "enable" if has_command else "disable"

    elif arg == "peer_type":
        value = "disable"
        if has_command_val:
            value = has_command_val.group("value").replace("-", "_")
    else:
        value = ""

        if has_command_val:
            value = has_command_val.group("value")

            if command in ["timers", "password"]:
                split_value = value.split()
                value = ""

                if arg in ["timers_keepalive", "pwd_type"]:
                    value = split_value[0]
                elif (
                    arg in ["timers_holdtime", "pwd"] and len(split_value) == 2
                ):
                    value = split_value[1]

    return value


def get_existing(module, args, warnings):
    existing = {}
    netcfg = CustomNetworkConfig(indent=2, contents=get_config(module))

    asn_regex = re.compile(
        r".*router\sbgp\s(?P<existing_asn>\d+(\.\d+)?).*", re.S
    )
    match_asn = asn_regex.match(str(netcfg))

    if match_asn:
        existing_asn = match_asn.group("existing_asn")
        parents = ["router bgp {0}".format(existing_asn)]

        if module.params["vrf"] != "default":
            parents.append("vrf {0}".format(module.params["vrf"]))

        parents.append("neighbor {0}".format(module.params["neighbor"]))
        config = netcfg.get_section(parents)
        if config:
            for arg in args:
                if arg not in ["asn", "vrf", "neighbor"]:
                    existing[arg] = get_value(arg, config)

            existing["asn"] = existing_asn
            existing["neighbor"] = module.params["neighbor"]
            existing["vrf"] = module.params["vrf"]
    else:
        warnings.append(
            "The BGP process didn't exist but the task" " just created it."
        )
    return existing


def apply_key_map(key_map, table):
    new_dict = {}
    for key in table:
        new_key = key_map.get(key)
        if new_key:
            new_dict[new_key] = table.get(key)

    return new_dict


def state_present(module, existing, proposed, candidate):
    commands = list()
    proposed_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, proposed)
    existing_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, existing)

    for key, value in proposed_commands.items():
        if value is True:
            commands.append(key)
        elif value is False:
            commands.append("no {0}".format(key))
        elif value == "default":
            if existing_commands.get(key):
                if key == "password":
                    commands.append("no password")
                else:
                    existing_value = existing_commands.get(key)
                    commands.append("no {0} {1}".format(key, existing_value))
        else:
            if key == "log-neighbor-changes":
                if value == "enable":
                    commands.append("{0}".format(key))
                elif value == "disable":
                    commands.append("{0} {1}".format(key, value))
                elif value == "inherit":
                    if existing_commands.get(key):
                        commands.append("no {0}".format(key))
            elif key == "password":
                pwd_type = module.params["pwd_type"]
                if pwd_type == "3des":
                    pwd_type = 3
                else:
                    pwd_type = 7
                command = "{0} {1} {2}".format(key, pwd_type, value)
                if command not in commands:
                    commands.append(command)
            elif key == "remove-private-as":
                if value == "enable":
                    command = "{0}".format(key)
                    commands.append(command)
                elif value == "disable":
                    if existing_commands.get(key) != "disable":
                        command = "no {0}".format(key)
                        commands.append(command)
                else:
                    command = "{0} {1}".format(key, value)
                    commands.append(command)
            elif key == "timers":
                if proposed["timers_keepalive"] != PARAM_TO_DEFAULT_KEYMAP.get(
                    "timers_keepalive"
                ) or proposed[
                    "timers_holdtime"
                ] != PARAM_TO_DEFAULT_KEYMAP.get(
                    "timers_holdtime"
                ):
                    command = "timers {0} {1}".format(
                        proposed["timers_keepalive"],
                        proposed["timers_holdtime"],
                    )
                    if command not in commands:
                        commands.append(command)
            elif key == "bfd":
                no_cmd = "no " if value == "disable" else ""
                commands.append(no_cmd + key)
            elif key == "peer-type":
                if value == "disable":
                    if existing_commands.get(key) != "disable":
                        command = "no {0}".format(key)
                        commands.append(command)
                elif value == "fabric_external":
                    ptype = "fabric-external"
                    command = "{0} {1}".format(key, ptype)
                    commands.append(command)
                elif value == "fabric_border_leaf":
                    ptype = "fabric-border-leaf"
                    command = "{0} {1}".format(key, ptype)
                    commands.append(command)
            else:
                command = "{0} {1}".format(key, value)
                commands.append(command)

    if commands:
        parents = ["router bgp {0}".format(module.params["asn"])]
        if module.params["vrf"] != "default":
            parents.append("vrf {0}".format(module.params["vrf"]))

        parents.append("neighbor {0}".format(module.params["neighbor"]))

        # make sure that local-as is the last command in the list.
        local_as_command = "local-as {0}".format(module.params["local_as"])
        if local_as_command in commands:
            commands.remove(local_as_command)
            commands.append(local_as_command)
        candidate.add(commands, parents=parents)


def state_absent(module, existing, proposed, candidate):
    commands = []
    parents = ["router bgp {0}".format(module.params["asn"])]
    if module.params["vrf"] != "default":
        parents.append("vrf {0}".format(module.params["vrf"]))

    commands.append("no neighbor {0}".format(module.params["neighbor"]))
    candidate.add(commands, parents=parents)


def main():
    argument_spec = dict(
        asn=dict(required=True, type="str"),
        vrf=dict(required=False, type="str", default="default"),
        neighbor=dict(required=True, type="str"),
        description=dict(required=False, type="str"),
        bfd=dict(required=False, type="str", choices=["enable", "disable"]),
        capability_negotiation=dict(required=False, type="bool"),
        connected_check=dict(required=False, type="bool"),
        dynamic_capability=dict(required=False, type="bool"),
        ebgp_multihop=dict(required=False, type="str"),
        local_as=dict(required=False, type="str"),
        log_neighbor_changes=dict(
            required=False,
            type="str",
            choices=["enable", "disable", "inherit"],
        ),
        low_memory_exempt=dict(required=False, type="bool"),
        maximum_peers=dict(required=False, type="str"),
        pwd=dict(required=False, type="str"),
        pwd_type=dict(
            required=False,
            type="str",
            choices=["3des", "cisco_type_7", "default"],
        ),
        remote_as=dict(required=False, type="str"),
        remove_private_as=dict(
            required=False,
            type="str",
            choices=["enable", "disable", "all", "replace-as"],
        ),
        shutdown=dict(required=False, type="bool"),
        suppress_4_byte_as=dict(required=False, type="bool"),
        timers_keepalive=dict(required=False, type="str"),
        timers_holdtime=dict(required=False, type="str"),
        transport_passive_only=dict(required=False, type="bool"),
        update_source=dict(required=False, type="str"),
        state=dict(
            choices=["present", "absent"], default="present", required=False
        ),
        peer_type=dict(
            required=False,
            type="str",
            choices=["disable", "fabric_border_leaf", "fabric_external"],
        ),
    )
    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_together=[
            ["timers_holdtime", "timers_keepalive"],
            ["pwd", "pwd_type"],
        ],
        supports_check_mode=True,
    )

    warnings = list()
    result = dict(changed=False, warnings=warnings)

    state = module.params["state"]

    if module.params["pwd_type"] == "default":
        module.params["pwd_type"] = "0"

    args = PARAM_TO_COMMAND_KEYMAP.keys()
    existing = get_existing(module, args, warnings)

    if existing.get("asn") and state == "present":
        if existing["asn"] != module.params["asn"]:
            module.fail_json(
                msg="Another BGP ASN already exists.",
                proposed_asn=module.params["asn"],
                existing_asn=existing.get("asn"),
            )

    proposed_args = dict(
        (k, v) for k, v in module.params.items() if v is not None and k in args
    )
    proposed = {}
    for key, value in proposed_args.items():
        if key not in ["asn", "vrf", "neighbor", "pwd_type"]:
            if str(value).lower() == "default":
                value = PARAM_TO_DEFAULT_KEYMAP.get(key, "default")
            if key == "bfd":
                if existing.get("bfd", "disable") != value:
                    proposed[key] = value
            elif existing.get(key) != value:
                proposed[key] = value

    candidate = CustomNetworkConfig(indent=3)
    if state == "present":
        state_present(module, existing, proposed, candidate)
    elif state == "absent" and existing:
        state_absent(module, existing, proposed, candidate)

    if candidate:
        candidate = candidate.items_text()
        if not module.check_mode:
            load_config(module, candidate)
        result["changed"] = True
        result["commands"] = candidate
    else:
        result["commands"] = []

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: icx_banner
author: "Ruckus Wireless (@Commscope)"
short_description: Manage multiline banners on Ruckus ICX 7000 series switches
description:
  - This will configure both login and motd banners on remote
    ruckus ICX 7000 series switches. It allows playbooks to add or remove
    banner text from the active running configuration.
notes:
  - Tested against ICX 10.1
options:
  banner:
    description:
      - Specifies which banner should be configured on the remote device.
    type: str
    required: true
    choices: ['motd', 'exec', 'incoming']
  text:
    description:
      - The banner text that should be
        present in the remote device running configuration.
        This argument accepts a multiline string, with no empty lines.
    type: str
  state:
    description:
      - Specifies whether or not the configuration is
        present in the current devices active running configuration.
    type: str
    default: present
    choices: ['present', 'absent']
  enterkey:
    description:
      - Specifies whether or not the motd configuration should accept
        the require-enter-key
      - Default is false.
    type: bool
  check_running_config:
    description:
      - Check running configuration. This can be set as environment variable.
       Module will use environment variable value(default:True), unless it is overridden,
       by specifying it as module parameter.
    type: bool
    default: yes

'''

EXAMPLES = """
- name: Configure the motd banner
  community.network.icx_banner:
    banner: motd
    text: |
        this is my motd banner
        that contains a multiline
        string
    state: present

- name: Remove the motd banner
  community.network.icx_banner:
    banner: motd
    state: absent

- name: Configure require-enter-key for motd
  community.network.icx_banner:
    banner: motd
    enterkey: True

- name: Remove require-enter-key for motd
  community.network.icx_banner:
    banner: motd
    enterkey: False
"""

RETURN = """
commands:
 description: The list of configuration mode commands to send to the device
 returned: always
 type: list
 sample:
    - banner motd
    - this is my motd banner
    - that contains a multiline
    - string
"""

import re
from ansible.module_utils._text import to_text
from ansible.module_utils.connection import exec_command
from ansible.module_utils.basic import AnsibleModule, env_fallback
from ansible_collections.community.network.plugins.module_utils.network.icx.icx import load_config, get_config
from ansible.module_utils.connection import Connection, ConnectionError


def map_obj_to_commands(updates, module):
    commands = list()
    state = module.params['state']
    want, have = updates

    if module.params['banner'] != 'motd' and module.params['enterkey']:
        module.fail_json(msg=module.params['banner'] + " banner can have text only, got enterkey")

    if state == 'absent':
        if 'text' in have.keys() and have['text']:
            commands.append('no banner %s' % module.params['banner'])
        if(module.params['enterkey'] is False):
            commands.append('no banner %s require-enter-key' % module.params['banner'])

    elif state == 'present':
        if module.params['text'] is None and module.params['enterkey'] is None:
            module.fail_json(msg=module.params['banner'] + " one of the following is required: text, enterkey:only if motd")

        if module.params["banner"] == "motd" and want['enterkey'] != have['enterkey']:
            if(module.params['enterkey']):
                commands.append('banner %s require-enter-key' % module.params['banner'])

        if want['text'] and (want['text'] != have.get('text')):
            module.params["enterkey"] = None
            banner_cmd = 'banner %s' % module.params['banner']
            banner_cmd += ' $\n'
            banner_cmd += module.params['text'].strip()
            banner_cmd += '\n$'
            commands.append(banner_cmd)
    return commands


def map_config_to_obj(module):
    compare = module.params.get('check_running_config')
    obj = {'banner': module.params['banner'], 'state': 'absent', 'enterkey': False}
    exec_command(module, 'skip')
    output_text = ''
    output_re = ''
    out = get_config(module, flags=['| begin banner %s'
                                    % module.params['banner']], compare=module.params['check_running_config'])
    if out:
        try:
            output_re = re.search(r'banner %s( require-enter-key)' % module.params['banner'], out, re.S).group(0)
            obj['enterkey'] = True
        except BaseException:
            pass
        try:
            output_text = re.search(r'banner %s (\$([^\$])+\$){1}' % module.params['banner'], out, re.S).group(1).strip('$\n')
        except BaseException:
            pass

    else:
        output_text = None
    if output_text:
        obj['text'] = output_text
        obj['state'] = 'present'
    if module.params['check_running_config'] is False:
        obj = {'banner': module.params['banner'], 'state': 'absent', 'enterkey': False, 'text': 'JUNK'}
    return obj


def map_params_to_obj(module):
    text = module.params['text']
    if text:
        text = str(text).strip()

    return {
        'banner': module.params['banner'],
        'text': text,
        'state': module.params['state'],
        'enterkey': module.params['enterkey']
    }


def main():
    """entry point for module execution
    """
    argument_spec = dict(
        banner=dict(required=True, choices=['motd', 'exec', 'incoming']),
        text=dict(),
        enterkey=dict(type='bool'),
        state=dict(default='present', choices=['present', 'absent']),
        check_running_config=dict(default=True, type='bool', fallback=(env_fallback, ['ANSIBLE_CHECK_ICX_RUNNING_CONFIG']))
    )

    required_one_of = [['text', 'enterkey', 'state']]
    module = AnsibleModule(argument_spec=argument_spec,
                           required_one_of=required_one_of,
                           supports_check_mode=True)

    warnings = list()
    results = {'changed': False}

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)
    commands = map_obj_to_commands((want, have), module)
    results['commands'] = commands

    if commands:
        if not module.check_mode:
            response = load_config(module, commands)

        results['changed'] = True

    module.exit_json(**results)


if __name__ == '__main__':
    main()



#!/usr/bin/python
""" PN-CLI vrouter-ospf-add/remove """

#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: pn_ospfarea
author: "Pluribus Networks (@amitsi)"
short_description: CLI command to add/remove ospf area to/from a vrouter.
deprecated:
  removed_in: 2.0.0  # was Ansible 2.12
  why: Doesn't support latest Pluribus Networks netvisor
  alternative: Latest modules will be pushed in Ansible future versions.
description:
  - Execute vrouter-ospf-add, vrouter-ospf-remove command.
  - This command adds/removes Open Shortest Path First(OSPF) area to/from
    a virtual router(vRouter) service.
options:
  pn_cliusername:
    description:
      - Login username.
    required: true
  pn_clipassword:
    description:
      - Login password.
    required: true
  pn_cliswitch:
    description:
      - Target switch(es) to run the CLI on.
    required: False
  state:
    description:
      - State the action to perform. Use 'present' to add ospf-area, 'absent'
        to remove ospf-area and 'update' to modify ospf-area.
    required: true
    choices: ['present', 'absent', 'update']
  pn_vrouter_name:
    description:
      - Specify the name of the vRouter.
    required: true
  pn_ospf_area:
    description:
      - Specify the OSPF area number.
    required: true
  pn_stub_type:
    description:
      - Specify the OSPF stub type.
    choices: ['none', 'stub', 'stub-no-summary', 'nssa', 'nssa-no-summary']
  pn_prefix_listin:
    description:
      - OSPF prefix list for filtering incoming packets.
  pn_prefix_listout:
    description:
      - OSPF prefix list for filtering outgoing packets.
  pn_quiet:
    description:
      - Enable/disable system information.
    required: false
    type: bool
    default: true
'''

EXAMPLES = """
- name: "Add OSPF area to vrouter"
  community.network.pn_ospfarea:
    state: present
    pn_cliusername: admin
    pn_clipassword: admin
    pn_ospf_area: 1.0.0.0
    pn_stub_type: stub

- name: "Remove OSPF from vrouter"
  pn_ospf:
    state: absent
    pn_cliusername: admin
    pn_clipassword: admin
    pn_vrouter_name: name-string
    pn_ospf_area: 1.0.0.0
"""

RETURN = """
command:
  description: The CLI command run on the target node(s).
  returned: always
  type: str
stdout:
  description: The set of responses from the ospf command.
  returned: always
  type: list
stderr:
  description: The set of error responses from the ospf command.
  returned: on error
  type: list
changed:
  description: Indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

import shlex

# AnsibleModule boilerplate
from ansible.module_utils.basic import AnsibleModule


def get_command_from_state(state):
    """
    This method gets appropriate command name for the state specified. It
    returns the command name for the specified state.
    :param state: The state for which the respective command name is required.
    """
    command = None
    if state == 'present':
        command = 'vrouter-ospf-area-add'
    if state == 'absent':
        command = 'vrouter-ospf-area-remove'
    if state == 'update':
        command = 'vrouter-ospf-area-modify'
    return command


def main():
    """ This section is for arguments parsing """
    module = AnsibleModule(
        argument_spec=dict(
            pn_cliusername=dict(required=True, type='str'),
            pn_clipassword=dict(required=True, type='str', no_log=True),
            pn_cliswitch=dict(required=False, type='str'),
            state=dict(required=True, type='str',
                       choices=['present', 'absent', 'update']),
            pn_vrouter_name=dict(required=True, type='str'),
            pn_ospf_area=dict(required=True, type='str'),
            pn_stub_type=dict(type='str', choices=['none', 'stub', 'nssa',
                                                   'stub-no-summary',
                                                   'nssa-no-summary']),
            pn_prefix_listin=dict(type='str'),
            pn_prefix_listout=dict(type='str'),
            pn_quiet=dict(type='bool', default='True')
        )
    )

    # Accessing the arguments
    cliusername = module.params['pn_cliusername']
    clipassword = module.params['pn_clipassword']
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    vrouter_name = module.params['pn_vrouter_name']
    ospf_area = module.params['pn_ospf_area']
    stub_type = module.params['pn_stub_type']
    prefix_listin = module.params['pn_prefix_listin']
    prefix_listout = module.params['pn_prefix_listout']
    quiet = module.params['pn_quiet']

    command = get_command_from_state(state)

    # Building the CLI command string
    cli = '/usr/bin/cli'

    if quiet is True:
        cli += ' --quiet '

    cli += ' --user %s:%s ' % (cliusername, clipassword)

    if cliswitch:
        if cliswitch == 'local':
            cli += ' switch-local '
        else:
            cli += ' switch ' + cliswitch

    cli += ' %s vrouter-name %s area %s ' % (command, vrouter_name, ospf_area)

    if stub_type:
        cli += ' stub-type ' + stub_type

    if prefix_listin:
        cli += ' prefix-list-in ' + prefix_listin

    if prefix_listout:
        cli += ' prefix-list-out ' + prefix_listout

    # Run the CLI command
    ospfcommand = shlex.split(cli)

    # 'out' contains the output
    # 'err' contains the error messages
    result, out, err = module.run_command(ospfcommand)

    # Response in JSON format
    if result != 0:
        module.exit_json(
            command=cli,
            stderr=err.rstrip("\r\n"),
            changed=False
        )

    else:
        module.exit_json(
            command=cli,
            stdout=out.rstrip("\r\n"),
            changed=True
        )


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# (c) 2016, techbizdev <techbizdev@paloaltonetworks.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: panos_security_rule
short_description: Create security rule policy on PAN-OS devices or Panorama management console.
description:
    - Security policies allow you to enforce rules and take action, and can be as general or specific as needed.
      The policy rules are compared against the incoming traffic in sequence, and because the first rule that matches the traffic is applied,
      the more specific rules must precede the more general ones.
author: "Ivan Bojer (@ivanbojer), Robert Hagen (@rnh556)"
requirements:
    - pan-python can be obtained from PyPI U(https://pypi.org/project/pan-python/)
    - pandevice can be obtained from PyPI U(https://pypi.org/project/pandevice/)
    - xmltodict can be obtained from PyPI U(https://pypi.org/project/xmltodict/)
deprecated:
    alternative: Use U(https://galaxy.ansible.com/PaloAltoNetworks/paloaltonetworks) instead.
    removed_in: 2.0.0  # was Ansible 2.12
    why: Consolidating code base.
notes:
    - Checkmode is not supported.
    - Panorama is supported.
options:
    ip_address:
        description:
            - IP address (or hostname) of PAN-OS device being configured.
        required: true
    username:
        description:
            - Username credentials to use for auth unless I(api_key) is set.
        default: "admin"
    password:
        description:
            - Password credentials to use for auth unless I(api_key) is set.
        required: true
    api_key:
        description:
            - API key that can be used instead of I(username)/I(password) credentials.
    operation:
        description:
            - The action to be taken.  Supported values are I(add)/I(update)/I(find)/I(delete).
        default: 'add'
        choices:
            - add
            - update
            - delete
            - find
    category:
        description:
            - The category.
        type: list
        default: ['any']
    rule_name:
        description:
            - Name of the security rule.
        required: true
    rule_type:
        description:
            - Type of security rule (version 6.1 of PanOS and above).
        default: "universal"
    description:
        description:
            - Description for the security rule.
    tag_name:
        description:
            - Administrative tags that can be added to the rule. Note, tags must be already defined.
    source_zone:
        description:
            - List of source zones.
        default: "any"
    destination_zone:
        description:
            - List of destination zones.
        default: "any"
    source_ip:
        description:
            - List of source addresses.
        default: "any"
    source_user:
        description:
            - Use users to enforce policy for individual users or a group of users.
        default: "any"
    hip_profiles:
        description: >
            - If you are using GlobalProtect with host information profile (HIP) enabled, you can also base the policy
            on information collected by GlobalProtect. For example, the user access level can be determined HIP that
            notifies the firewall about the user's local configuration.
        default: "any"
    destination_ip:
        description:
            - List of destination addresses.
        default: "any"
    application:
        description:
            - List of applications.
        default: "any"
    service:
        description:
            - List of services.
        default: "application-default"
    log_start:
        description:
            - Whether to log at session start.
        type: bool
        default: false
    log_end:
        description:
            - Whether to log at session end.
        default: true
        type: bool
    action:
        description:
            - Action to apply once rules maches.
        default: "allow"
    group_profile:
        description: >
            - Security profile group that is already defined in the system. This property supersedes antivirus,
            vulnerability, spyware, url_filtering, file_blocking, data_filtering, and wildfire_analysis properties.
    antivirus:
        description:
            - Name of the already defined antivirus profile.
    vulnerability:
        description:
            - Name of the already defined vulnerability profile.
    spyware:
        description:
            - Name of the already defined spyware profile.
    url_filtering:
        description:
            - Name of the already defined url_filtering profile.
    file_blocking:
        description:
            - Name of the already defined file_blocking profile.
    data_filtering:
        description:
            - Name of the already defined data_filtering profile.
    wildfire_analysis:
        description:
            - Name of the already defined wildfire_analysis profile.
    devicegroup:
        description: >
            - Device groups are used for the Panorama interaction with Firewall(s). The group must exists on Panorama.
            If device group is not define we assume that we are contacting Firewall.
    commit:
        description:
            - Commit configuration if changed.
        type: bool
        default: 'yes'
'''

EXAMPLES = '''
- name: Add an SSH inbound rule to devicegroup
  community.network.panos_security_rule:
    ip_address: '{{ ip_address }}'
    username: '{{ username }}'
    password: '{{ password }}'
    operation: 'add'
    rule_name: 'SSH permit'
    description: 'SSH rule test'
    tag_name: ['ProjectX']
    source_zone: ['public']
    destination_zone: ['private']
    source_ip: ['any']
    source_user: ['any']
    destination_ip: ['1.1.1.1']
    category: ['any']
    application: ['ssh']
    service: ['application-default']
    hip_profiles: ['any']
    action: 'allow'
    devicegroup: 'Cloud Edge'

- name: Add a rule to allow HTTP multimedia only from CDNs
  community.network.panos_security_rule:
    ip_address: '10.5.172.91'
    username: 'admin'
    password: 'paloalto'
    operation: 'add'
    rule_name: 'HTTP Multimedia'
    description: 'Allow HTTP multimedia only to host at 1.1.1.1'
    source_zone: ['public']
    destination_zone: ['private']
    source_ip: ['any']
    source_user: ['any']
    destination_ip: ['1.1.1.1']
    category: ['content-delivery-networks']
    application: ['http-video', 'http-audio']
    service: ['service-http', 'service-https']
    hip_profiles: ['any']
    action: 'allow'

- name: Add a more complex rule that uses security profiles
  community.network.panos_security_rule:
    ip_address: '{{ ip_address }}'
    username: '{{ username }}'
    password: '{{ password }}'
    operation: 'add'
    rule_name: 'Allow HTTP w profile'
    log_start: false
    log_end: true
    action: 'allow'
    antivirus: 'default'
    vulnerability: 'default'
    spyware: 'default'
    url_filtering: 'default'
    wildfire_analysis: 'default'

- name: Delete a devicegroup security rule
  community.network.panos_security_rule:
    ip_address: '{{ ip_address }}'
    api_key: '{{ api_key }}'
    operation: 'delete'
    rule_name: 'Allow telnet'
    devicegroup: 'DC Firewalls'

- name: Find a specific security rule
  community.network.panos_security_rule:
    ip_address: '{{ ip_address }}'
    password: '{{ password }}'
    operation: 'find'
    rule_name: 'Allow RDP to DCs'
  register: result
- ansible.builtin.debug: msg='{{result.stdout_lines}}'

'''

RETURN = '''
# Default return values
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native

try:
    import pan.xapi
    from pan.xapi import PanXapiError
    import pandevice
    from pandevice import base
    from pandevice import firewall
    from pandevice import panorama
    from pandevice import objects
    from pandevice import policies
    import xmltodict
    import json

    HAS_LIB = True
except ImportError:
    HAS_LIB = False


def get_devicegroup(device, devicegroup):
    dg_list = device.refresh_devices()
    for group in dg_list:
        if isinstance(group, pandevice.panorama.DeviceGroup):
            if group.name == devicegroup:
                return group
    return False


def get_rulebase(device, devicegroup):
    # Build the rulebase
    if isinstance(device, pandevice.firewall.Firewall):
        rulebase = pandevice.policies.Rulebase()
        device.add(rulebase)
    elif isinstance(device, pandevice.panorama.Panorama):
        dg = panorama.DeviceGroup(devicegroup)
        device.add(dg)
        rulebase = policies.PreRulebase()
        dg.add(rulebase)
    else:
        return False
    policies.SecurityRule.refreshall(rulebase)
    return rulebase


def find_rule(rulebase, rule_name):
    # Search for the rule name
    rule = rulebase.find(rule_name)
    if rule:
        return rule
    else:
        return False


def rule_is_match(propose_rule, current_rule):

    match_check = ['name', 'description', 'group_profile', 'antivirus', 'vulnerability',
                   'spyware', 'url_filtering', 'file_blocking', 'data_filtering',
                   'wildfire_analysis', 'type', 'action', 'tag', 'log_start', 'log_end']
    list_check = ['tozone', 'fromzone', 'source', 'source_user', 'destination', 'category',
                  'application', 'service', 'hip_profiles']

    for check in match_check:
        propose_check = getattr(propose_rule, check, None)
        current_check = getattr(current_rule, check, None)
        if propose_check != current_check:
            return False
    for check in list_check:
        propose_check = getattr(propose_rule, check, [])
        current_check = getattr(current_rule, check, [])
        if set(propose_check) != set(current_check):
            return False
    return True


def create_security_rule(**kwargs):
    security_rule = policies.SecurityRule(
        name=kwargs['rule_name'],
        description=kwargs['description'],
        fromzone=kwargs['source_zone'],
        source=kwargs['source_ip'],
        source_user=kwargs['source_user'],
        hip_profiles=kwargs['hip_profiles'],
        tozone=kwargs['destination_zone'],
        destination=kwargs['destination_ip'],
        application=kwargs['application'],
        service=kwargs['service'],
        category=kwargs['category'],
        log_start=kwargs['log_start'],
        log_end=kwargs['log_end'],
        action=kwargs['action'],
        type=kwargs['rule_type']
    )

    if 'tag_name' in kwargs:
        security_rule.tag = kwargs['tag_name']

    # profile settings
    if 'group_profile' in kwargs:
        security_rule.group = kwargs['group_profile']
    else:
        if 'antivirus' in kwargs:
            security_rule.virus = kwargs['antivirus']
        if 'vulnerability' in kwargs:
            security_rule.vulnerability = kwargs['vulnerability']
        if 'spyware' in kwargs:
            security_rule.spyware = kwargs['spyware']
        if 'url_filtering' in kwargs:
            security_rule.url_filtering = kwargs['url_filtering']
        if 'file_blocking' in kwargs:
            security_rule.file_blocking = kwargs['file_blocking']
        if 'data_filtering' in kwargs:
            security_rule.data_filtering = kwargs['data_filtering']
        if 'wildfire_analysis' in kwargs:
            security_rule.wildfire_analysis = kwargs['wildfire_analysis']
    return security_rule


def add_rule(rulebase, sec_rule):
    if rulebase:
        rulebase.add(sec_rule)
        sec_rule.create()
        return True
    else:
        return False


def update_rule(rulebase, nat_rule):
    if rulebase:
        rulebase.add(nat_rule)
        nat_rule.apply()
        return True
    else:
        return False


def main():
    argument_spec = dict(
        ip_address=dict(required=True),
        password=dict(no_log=True),
        username=dict(default='admin'),
        api_key=dict(no_log=True),
        operation=dict(default='add', choices=['add', 'update', 'delete', 'find']),
        rule_name=dict(required=True),
        description=dict(default=''),
        tag_name=dict(type='list'),
        destination_zone=dict(type='list', default=['any']),
        source_zone=dict(type='list', default=['any']),
        source_ip=dict(type='list', default=["any"]),
        source_user=dict(type='list', default=['any']),
        destination_ip=dict(type='list', default=["any"]),
        category=dict(type='list', default=['any']),
        application=dict(type='list', default=['any']),
        service=dict(type='list', default=['application-default']),
        hip_profiles=dict(type='list', default=['any']),
        group_profile=dict(),
        antivirus=dict(),
        vulnerability=dict(),
        spyware=dict(),
        url_filtering=dict(),
        file_blocking=dict(),
        data_filtering=dict(),
        wildfire_analysis=dict(),
        log_start=dict(type='bool', default=False),
        log_end=dict(type='bool', default=True),
        rule_type=dict(default='universal'),
        action=dict(default='allow'),
        devicegroup=dict(),
        commit=dict(type='bool', default=True)
    )
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=False,
                           required_one_of=[['api_key', 'password']])
    if not HAS_LIB:
        module.fail_json(msg='Missing required libraries.')

    ip_address = module.params["ip_address"]
    password = module.params["password"]
    username = module.params['username']
    api_key = module.params['api_key']
    operation = module.params['operation']
    rule_name = module.params['rule_name']
    description = module.params['description']
    tag_name = module.params['tag_name']
    source_zone = module.params['source_zone']
    source_ip = module.params['source_ip']
    source_user = module.params['source_user']
    hip_profiles = module.params['hip_profiles']
    destination_zone = module.params['destination_zone']
    destination_ip = module.params['destination_ip']
    application = module.params['application']
    service = module.params['service']
    category = module.params['category']
    log_start = module.params['log_start']
    log_end = module.params['log_end']
    action = module.params['action']
    group_profile = module.params['group_profile']
    antivirus = module.params['antivirus']
    vulnerability = module.params['vulnerability']
    spyware = module.params['spyware']
    url_filtering = module.params['url_filtering']
    file_blocking = module.params['file_blocking']
    data_filtering = module.params['data_filtering']
    wildfire_analysis = module.params['wildfire_analysis']
    rule_type = module.params['rule_type']
    devicegroup = module.params['devicegroup']

    commit = module.params['commit']

    # Create the device with the appropriate pandevice type
    device = base.PanDevice.create_from_device(ip_address, username, password, api_key=api_key)

    # If Panorama, validate the devicegroup
    dev_group = None
    if devicegroup and isinstance(device, panorama.Panorama):
        dev_group = get_devicegroup(device, devicegroup)
        if dev_group:
            device.add(dev_group)
        else:
            module.fail_json(msg='\'%s\' device group not found in Panorama. Is the name correct?' % devicegroup)

    # Get the rulebase
    rulebase = get_rulebase(device, dev_group)

    # Which action shall we take on the object?
    if operation == "find":
        # Search for the object
        match = find_rule(rulebase, rule_name)
        # If found, format and return the result
        if match:
            match_dict = xmltodict.parse(match.element_str())
            module.exit_json(
                stdout_lines=json.dumps(match_dict, indent=2),
                msg='Rule matched'
            )
        else:
            module.fail_json(msg='Rule \'%s\' not found. Is the name correct?' % rule_name)
    elif operation == "delete":
        # Search for the object
        match = find_rule(rulebase, rule_name)
        # If found, delete it
        if match:
            try:
                if commit:
                    match.delete()
            except PanXapiError as exc:
                module.fail_json(msg=to_native(exc))

            module.exit_json(changed=True, msg='Rule \'%s\' successfully deleted' % rule_name)
        else:
            module.fail_json(msg='Rule \'%s\' not found. Is the name correct?' % rule_name)
    elif operation == "add":
        new_rule = create_security_rule(
            rule_name=rule_name,
            description=description,
            tag_name=tag_name,
            source_zone=source_zone,
            destination_zone=destination_zone,
            source_ip=source_ip,
            source_user=source_user,
            destination_ip=destination_ip,
            category=category,
            application=application,
            service=service,
            hip_profiles=hip_profiles,
            group_profile=group_profile,
            antivirus=antivirus,
            vulnerability=vulnerability,
            spyware=spyware,
            url_filtering=url_filtering,
            file_blocking=file_blocking,
            data_filtering=data_filtering,
            wildfire_analysis=wildfire_analysis,
            log_start=log_start,
            log_end=log_end,
            rule_type=rule_type,
            action=action
        )
        # Search for the rule. Fail if found.
        match = find_rule(rulebase, rule_name)
        if match:
            if rule_is_match(match, new_rule):
                module.exit_json(changed=False, msg='Rule \'%s\' is already in place' % rule_name)
            else:
                module.fail_json(msg='Rule \'%s\' already exists. Use operation: \'update\' to change it.' % rule_name)
        else:
            try:
                changed = add_rule(rulebase, new_rule)
                if changed and commit:
                    device.commit(sync=True)
            except PanXapiError as exc:
                module.fail_json(msg=to_native(exc))
            module.exit_json(changed=changed, msg='Rule \'%s\' successfully added' % rule_name)
    elif operation == 'update':
        # Search for the rule. Update if found.
        match = find_rule(rulebase, rule_name)
        if match:
            try:
                new_rule = create_security_rule(
                    rule_name=rule_name,
                    description=description,
                    tag_name=tag_name,
                    source_zone=source_zone,
                    destination_zone=destination_zone,
                    source_ip=source_ip,
                    source_user=source_user,
                    destination_ip=destination_ip,
                    category=category,
                    application=application,
                    service=service,
                    hip_profiles=hip_profiles,
                    group_profile=group_profile,
                    antivirus=antivirus,
                    vulnerability=vulnerability,
                    spyware=spyware,
                    url_filtering=url_filtering,
                    file_blocking=file_blocking,
                    data_filtering=data_filtering,
                    wildfire_analysis=wildfire_analysis,
                    log_start=log_start,
                    log_end=log_end,
                    rule_type=rule_type,
                    action=action
                )
                changed = update_rule(rulebase, new_rule)
                if changed and commit:
                    device.commit(sync=True)
            except PanXapiError as exc:
                module.fail_json(msg=to_native(exc))
            module.exit_json(changed=changed, msg='Rule \'%s\' successfully updated' % rule_name)
        else:
            module.fail_json(msg='Rule \'%s\' does not exist. Use operation: \'add\' to add it.' % rule_name)


if __name__ == '__main__':
    main()



# Copyright (c) 2018 Cisco and/or its affiliates.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import re

from ansible.module_utils._text import to_text
from ansible.module_utils.common.collections import is_string
from ansible.module_utils.six import iteritems

INVALID_IDENTIFIER_SYMBOLS = r'[^a-zA-Z0-9_]'

IDENTITY_PROPERTIES = ['id', 'version', 'ruleId']
NON_COMPARABLE_PROPERTIES = IDENTITY_PROPERTIES + ['isSystemDefined', 'links', 'token', 'rulePosition']


class HTTPMethod:
    GET = 'get'
    POST = 'post'
    PUT = 'put'
    DELETE = 'delete'


class ResponseParams:
    SUCCESS = 'success'
    STATUS_CODE = 'status_code'
    RESPONSE = 'response'


class FtdConfigurationError(Exception):
    def __init__(self, msg, obj=None):
        super(FtdConfigurationError, self).__init__(msg)
        self.msg = msg
        self.obj = obj


class FtdServerError(Exception):
    def __init__(self, response, code):
        super(FtdServerError, self).__init__(response)
        self.response = response
        self.code = code


class FtdUnexpectedResponse(Exception):
    """The exception to be raised in case of unexpected responses from 3d parties."""
    pass


def construct_ansible_facts(response, params):
    facts = dict()
    if response:
        response_body = response['items'] if 'items' in response else response
        if params.get('register_as'):
            facts[params['register_as']] = response_body
        elif type(response_body) is dict and response_body.get('name') and response_body.get('type'):
            object_name = re.sub(INVALID_IDENTIFIER_SYMBOLS, '_', response_body['name'].lower())
            fact_name = '%s_%s' % (response_body['type'], object_name)
            facts[fact_name] = response_body
    return facts


def copy_identity_properties(source_obj, dest_obj):
    for property_name in IDENTITY_PROPERTIES:
        if property_name in source_obj:
            dest_obj[property_name] = source_obj[property_name]
    return dest_obj


def is_object_ref(d):
    """
    Checks if a dictionary is a reference object. The dictionary is considered to be a
    reference object when it contains non-empty 'id' and 'type' fields.

    :type d: dict
    :return: True if passed dictionary is a reference object, otherwise False
    """
    has_id = 'id' in d.keys() and d['id']
    has_type = 'type' in d.keys() and d['type']
    return has_id and has_type


def equal_object_refs(d1, d2):
    """
    Checks whether two references point to the same object.

    :type d1: dict
    :type d2: dict
    :return: True if passed references point to the same object, otherwise False
    """
    have_equal_ids = d1['id'] == d2['id']
    have_equal_types = d1['type'] == d2['type']
    return have_equal_ids and have_equal_types


def equal_lists(l1, l2):
    """
    Checks whether two lists are equal. The order of elements in the arrays is important.

    :type l1: list
    :type l2: list
    :return: True if passed lists, their elements and order of elements are equal. Otherwise, returns False.
    """
    if len(l1) != len(l2):
        return False

    for v1, v2 in zip(l1, l2):
        if not equal_values(v1, v2):
            return False

    return True


def equal_dicts(d1, d2, compare_by_reference=True):
    """
    Checks whether two dictionaries are equal. If `compare_by_reference` is set to True, dictionaries referencing
    objects are compared using `equal_object_refs` method. Otherwise, every key and value is checked.

    :type d1: dict
    :type d2: dict
    :param compare_by_reference: if True, dictionaries referencing objects are compared using `equal_object_refs` method
    :return: True if passed dicts are equal. Otherwise, returns False.
    """
    if compare_by_reference and is_object_ref(d1) and is_object_ref(d2):
        return equal_object_refs(d1, d2)

    if len(d1) != len(d2):
        return False

    for key, v1 in d1.items():
        if key not in d2:
            return False

        v2 = d2[key]
        if not equal_values(v1, v2):
            return False

    return True


def equal_values(v1, v2):
    """
    Checks whether types and content of two values are the same. In case of complex objects, the method might be
    called recursively.

    :param v1: first value
    :param v2: second value
    :return: True if types and content of passed values are equal. Otherwise, returns False.
    :rtype: bool
    """

    # string-like values might have same text but different types, so checking them separately
    if is_string(v1) and is_string(v2):
        return to_text(v1) == to_text(v2)

    if type(v1) != type(v2):
        return False
    value_type = type(v1)

    if value_type == list:
        return equal_lists(v1, v2)
    elif value_type == dict:
        return equal_dicts(v1, v2)
    else:
        return v1 == v2


def equal_objects(d1, d2):
    """
    Checks whether two objects are equal. Ignores special object properties (e.g. 'id', 'version') and
    properties with None and empty values. In case properties contains a reference to the other object,
    only object identities (ids and types) are checked. Also, if an array field contains multiple references
    to the same object, duplicates are ignored when comparing objects.

    :type d1: dict
    :type d2: dict
    :return: True if passed objects and their properties are equal. Otherwise, returns False.
    """

    def prepare_data_for_comparison(d):
        d = dict((k, d[k]) for k in d.keys() if k not in NON_COMPARABLE_PROPERTIES and d[k])
        d = delete_ref_duplicates(d)
        return d

    d1 = prepare_data_for_comparison(d1)
    d2 = prepare_data_for_comparison(d2)
    return equal_dicts(d1, d2, compare_by_reference=False)


def delete_ref_duplicates(d):
    """
    Removes reference duplicates from array fields: if an array contains multiple items and some of
    them refer to the same object, only unique references are preserved (duplicates are removed).

    :param d: dict with data
    :type d: dict
    :return: dict without reference duplicates
    """

    def delete_ref_duplicates_from_list(refs):
        if all(type(i) == dict and is_object_ref(i) for i in refs):
            unique_refs = set()
            unique_list = list()
            for i in refs:
                key = (i['id'], i['type'])
                if key not in unique_refs:
                    unique_refs.add(key)
                    unique_list.append(i)

            return list(unique_list)

        else:
            return refs

    if not d:
        return d

    modified_d = {}
    for k, v in iteritems(d):
        if type(v) == list:
            modified_d[k] = delete_ref_duplicates_from_list(v)
        elif type(v) == dict:
            modified_d[k] = delete_ref_duplicates(v)
        else:
            modified_d[k] = v
    return modified_d



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.arista.eos.tests.unit.compat.mock import patch
from ansible_collections.arista.eos.plugins.modules import eos_ospfv3
from ansible_collections.arista.eos.tests.unit.modules.utils import (
    set_module_args,
)
from .eos_module import TestEosModule, load_fixture


class TestEosOspfv3Module(TestEosModule):
    module = eos_ospfv3

    def setUp(self):
        super(TestEosOspfv3Module, self).setUp()

        self.mock_get_resource_connection_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.resource_module.get_resource_connection"
        )
        self.get_resource_connection_config = (
            self.mock_get_resource_connection_config.start()
        )

        self.mock_execute_show_command = patch(
            "ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.ospfv3.ospfv3.Ospfv3Facts.get_config"
        )
        self.execute_show_command = self.mock_execute_show_command.start()

    def tearDown(self):
        super(TestEosOspfv3Module, self).tearDown()
        self.mock_get_resource_connection_config.stop()
        self.mock_execute_show_command.stop()

    def load_fixtures(self, commands=None, transport="cli", filename=None):
        if filename is None:
            filename = "eos_ospfv3_config.cfg"

        def load_from_file(*args, **kwargs):
            output = load_fixture(filename)
            return output

        self.execute_show_command.side_effect = load_from_file

    def test_eos_ospfv3_merged_idempotent(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[
                        dict(
                            vrf="default",
                            adjacency=dict(exchange_start=dict(threshold=11)),
                            areas=[
                                dict(stub=dict(set=True), area_id="0.0.0.20"),
                                dict(
                                    area_id="0.0.0.40",
                                    stub=dict(set=True),
                                    default_cost=45,
                                ),
                            ],
                            timers=dict(pacing=7),
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    fips_restrictions=True,
                                    redistribute=[dict(routes="connected")],
                                ),
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    router_id="10.1.1.1",
                                ),
                            ],
                        ),
                        dict(
                            vrf="vrf01",
                            bfd=dict(all_interfaces=True),
                            log_adjacency_changes=dict(detail=True),
                            areas=[
                                dict(
                                    area_id="0.0.0.0",
                                    encryption=dict(
                                        algorithm="sha1",
                                        hidden_key=True,
                                        passphrase="7hl8FV3lZ6H1mAKpjL47hQ==",
                                        encryption="null",
                                        spi=44,
                                    ),
                                )
                            ],
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    fips_restrictions=True,
                                    redistribute=[
                                        dict(
                                            routes="connected",
                                            route_map="MAP01",
                                        )
                                    ],
                                    passive_interface=True,
                                    maximum_paths=100,
                                ),
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    areas=[
                                        dict(
                                            area_id="0.0.0.10",
                                            nssa=dict(no_summary=True),
                                        )
                                    ],
                                    default_information=dict(
                                        originate=True,
                                        route_map="DefaultRouteFilter",
                                    ),
                                    max_metric=dict(
                                        router_lsa=dict(
                                            external_lsa=dict(
                                                max_metric_value=25
                                            ),
                                            summary_lsa=dict(set=True),
                                        )
                                    ),
                                ),
                            ],
                        ),
                        dict(
                            vrf="vrf02",
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    areas=[
                                        dict(
                                            area_id="0.0.0.1",
                                            stub=dict(set=True),
                                        )
                                    ],
                                    distance=200,
                                    router_id="10.17.0.3",
                                    timers=dict(
                                        out_delay=10,
                                        throttle=dict(
                                            initial=56,
                                            max=56,
                                            min=56,
                                            spf=True,
                                        ),
                                    ),
                                )
                            ],
                        ),
                    ]
                ),
                state="merged",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_eos_ospfv3_merged(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[
                        dict(
                            vrf="default",
                            areas=[
                                dict(
                                    area_id="0.0.0.20",
                                    authentication=dict(
                                        algorithm="sha1",
                                        spi="33",
                                        hidden_key=True,
                                        passphrase="4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
                                    ),
                                )
                            ],
                            timers=dict(pacing=7),
                        ),
                        dict(
                            vrf="vrf03",
                            log_adjacency_changes=dict(detail=True),
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv6",
                                    areas=[
                                        dict(
                                            area_id="0.0.0.43",
                                            nssa=dict(no_summary=True),
                                            ranges=[
                                                dict(
                                                    address="20.1.1.0/24",
                                                    advertise=False,
                                                )
                                            ],
                                        )
                                    ],
                                    default_information=dict(
                                        originate=True,
                                        route_map="DefaultRouteFilter",
                                    ),
                                    graceful_restart=dict(set=True),
                                )
                            ],
                        ),
                    ]
                ),
                state="merged",
            )
        )
        commands = [
            "router ospfv3",
            "area 0.0.0.20 authentication ipsec spi 33 sha1 passphrase 7 4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
            "exit",
            "router ospfv3 vrf vrf03",
            "fips restrictions",
            "log-adjacency-changes detail",
            "address-family ipv6",
            "area 0.0.0.43 nssa no-summary",
            "area 0.0.0.43 range 20.1.1.0/24 not-advertise",
            "default-information originate route-map DefaultRouteFilter",
            "graceful-restart",
            "exit",
            "exit",
        ]
        self.execute_module(changed=True, commands=commands)

    def test_eos_ospfv3_replaced(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[
                        dict(
                            vrf="vrf02",
                            areas=[
                                dict(
                                    area_id="0.0.0.20",
                                    authentication=dict(
                                        algorithm="sha1",
                                        spi="33",
                                        hidden_key=True,
                                        passphrase="4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
                                    ),
                                )
                            ],
                            log_adjacency_changes=dict(detail=True),
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv6",
                                    areas=[
                                        dict(
                                            area_id="0.0.0.43",
                                            nssa=dict(no_summary=True),
                                            ranges=[
                                                dict(
                                                    address="20.1.1.0/24",
                                                    advertise=False,
                                                )
                                            ],
                                        )
                                    ],
                                    default_information=dict(
                                        originate=True,
                                        route_map="DefaultRouteFilter",
                                    ),
                                    graceful_restart=dict(set=True),
                                )
                            ],
                        )
                    ]
                ),
                state="replaced",
            )
        )
        commands = [
            "router ospfv3 vrf vrf02",
            "address-family ipv6",
            "no area 0.0.0.1 stub",
            "no distance ospf intra-area 200",
            "no fips restrictions",
            "no router-id",
            "no timers out-delay 10",
            "no timers throttle spf 56 56 56",
            "area 0.0.0.43 nssa no-summary",
            "area 0.0.0.43 range 20.1.1.0/24 not-advertise",
            "default-information originate route-map DefaultRouteFilter",
            "graceful-restart",
            "exit",
            "area 0.0.0.20 authentication ipsec spi 33 sha1 passphrase 7 4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
            "log-adjacency-changes detail",
            "exit",
        ]
        self.execute_module(changed=True, commands=commands)

    def test_eos_ospfv3_replaced_idempotent(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[
                        dict(
                            vrf="vrf01",
                            bfd=dict(all_interfaces=True),
                            log_adjacency_changes=dict(detail=True),
                            areas=[
                                dict(
                                    area_id="0.0.0.0",
                                    encryption=dict(
                                        algorithm="sha1",
                                        hidden_key=True,
                                        passphrase="7hl8FV3lZ6H1mAKpjL47hQ==",
                                        encryption="null",
                                        spi=44,
                                    ),
                                )
                            ],
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    fips_restrictions=True,
                                    redistribute=[
                                        dict(
                                            routes="connected",
                                            route_map="MAP01",
                                        )
                                    ],
                                    passive_interface=True,
                                    maximum_paths=100,
                                ),
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    areas=[
                                        dict(
                                            area_id="0.0.0.10",
                                            nssa=dict(no_summary=True),
                                        )
                                    ],
                                    default_information=dict(
                                        originate=True,
                                        route_map="DefaultRouteFilter",
                                    ),
                                    max_metric=dict(
                                        router_lsa=dict(
                                            external_lsa=dict(
                                                max_metric_value=25
                                            ),
                                            summary_lsa=dict(set=True),
                                        )
                                    ),
                                ),
                            ],
                        )
                    ]
                ),
                state="replaced",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_eos_ospfv3_overridden_idempotent(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[
                        dict(
                            vrf="default",
                            adjacency=dict(exchange_start=dict(threshold=11)),
                            areas=[
                                dict(stub=dict(set=True), area_id="0.0.0.20"),
                                dict(
                                    area_id="0.0.0.40",
                                    stub=dict(set=True),
                                    default_cost=45,
                                ),
                            ],
                            timers=dict(pacing=7),
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    fips_restrictions=True,
                                    redistribute=[dict(routes="connected")],
                                ),
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    router_id="10.1.1.1",
                                ),
                            ],
                        ),
                        dict(
                            vrf="vrf01",
                            bfd=dict(all_interfaces=True),
                            log_adjacency_changes=dict(detail=True),
                            areas=[
                                dict(
                                    area_id="0.0.0.0",
                                    encryption=dict(
                                        algorithm="sha1",
                                        hidden_key=True,
                                        passphrase="7hl8FV3lZ6H1mAKpjL47hQ==",
                                        encryption="null",
                                        spi=44,
                                    ),
                                )
                            ],
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    fips_restrictions=True,
                                    redistribute=[
                                        dict(
                                            routes="connected",
                                            route_map="MAP01",
                                        )
                                    ],
                                    passive_interface=True,
                                    maximum_paths=100,
                                ),
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    areas=[
                                        dict(
                                            area_id="0.0.0.10",
                                            nssa=dict(no_summary=True),
                                        )
                                    ],
                                    default_information=dict(
                                        originate=True,
                                        route_map="DefaultRouteFilter",
                                    ),
                                    max_metric=dict(
                                        router_lsa=dict(
                                            external_lsa=dict(
                                                max_metric_value=25
                                            ),
                                            summary_lsa=dict(set=True),
                                        )
                                    ),
                                ),
                            ],
                        ),
                        dict(
                            vrf="vrf02",
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    areas=[
                                        dict(
                                            area_id="0.0.0.1",
                                            stub=dict(set=True),
                                        )
                                    ],
                                    distance=200,
                                    router_id="10.17.0.3",
                                    timers=dict(
                                        out_delay=10,
                                        throttle=dict(
                                            initial=56,
                                            max=56,
                                            min=56,
                                            spf=True,
                                        ),
                                    ),
                                )
                            ],
                        ),
                    ]
                ),
                state="overridden",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_eos_ospfv3_overridden(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[
                        dict(
                            vrf="default",
                            adjacency=dict(exchange_start=dict(threshold=11)),
                            areas=[
                                dict(stub=dict(set=True), area_id="0.0.0.20"),
                                dict(
                                    area_id="0.0.0.40",
                                    stub=dict(set=True),
                                    default_cost=45,
                                ),
                            ],
                            timers=dict(pacing=7),
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    fips_restrictions=True,
                                    redistribute=[dict(routes="connected")],
                                ),
                                dict(
                                    afi="ipv6",
                                    fips_restrictions=True,
                                    router_id="10.1.1.1",
                                ),
                            ],
                        ),
                        dict(
                            vrf="vrf02",
                            areas=[
                                dict(
                                    area_id="0.0.0.20",
                                    authentication=dict(
                                        algorithm="sha1",
                                        spi="33",
                                        hidden_key=True,
                                        passphrase="4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
                                    ),
                                )
                            ],
                            log_adjacency_changes=dict(detail=True),
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv6",
                                    areas=[
                                        dict(
                                            area_id="0.0.0.43",
                                            nssa=dict(no_summary=True),
                                            ranges=[
                                                dict(
                                                    address="20.1.1.0/24",
                                                    advertise=False,
                                                )
                                            ],
                                        )
                                    ],
                                    default_information=dict(
                                        originate=True,
                                        route_map="DefaultRouteFilter",
                                    ),
                                    graceful_restart=dict(set=True),
                                )
                            ],
                        ),
                    ]
                ),
                state="overridden",
            )
        )
        commands = [
            "router ospfv3 vrf vrf02",
            "address-family ipv6",
            "no area 0.0.0.1 stub",
            "no distance ospf intra-area 200",
            "no fips restrictions",
            "no router-id",
            "no timers out-delay 10",
            "no timers throttle spf 56 56 56",
            "area 0.0.0.43 nssa no-summary",
            "area 0.0.0.43 range 20.1.1.0/24 not-advertise",
            "default-information originate route-map DefaultRouteFilter",
            "graceful-restart",
            "exit",
            "area 0.0.0.20 authentication ipsec spi 33 sha1 passphrase 7 4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
            "log-adjacency-changes detail",
            "exit",
            "no router ospfv3 vrf vrf01",
        ]

        self.execute_module(changed=True, commands=commands)

    def test_eos_ospfv3_deleted(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[dict(vrf="default"), dict(vrf="vrf02")]
                ),
                state="deleted",
            )
        )
        commands = ["no router ospfv3 vrf vrf02", "no router ospfv3"]

        self.execute_module(changed=True, commands=commands)

    def test_eos_ospfv3_rendered(self):
        set_module_args(
            dict(
                config=dict(
                    processes=[
                        dict(
                            vrf="default",
                            areas=[
                                dict(
                                    area_id="0.0.0.20",
                                    authentication=dict(
                                        algorithm="sha1",
                                        spi="33",
                                        hidden_key=True,
                                        passphrase="4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
                                    ),
                                )
                            ],
                            timers=dict(pacing=7),
                        ),
                        dict(
                            vrf="vrf03",
                            log_adjacency_changes=dict(detail=True),
                            fips_restrictions=True,
                            address_family=[
                                dict(
                                    afi="ipv6",
                                    areas=[
                                        dict(
                                            area_id="0.0.0.43",
                                            nssa=dict(no_summary=True),
                                            ranges=[
                                                dict(
                                                    address="20.1.1.0/24",
                                                    advertise=False,
                                                )
                                            ],
                                        )
                                    ],
                                    default_information=dict(
                                        originate=True,
                                        route_map="DefaultRouteFilter",
                                    ),
                                    graceful_restart=dict(set=True),
                                )
                            ],
                        ),
                    ]
                ),
                state="rendered",
            )
        )
        commands = [
            "router ospfv3",
            "area 0.0.0.20 authentication ipsec spi 33 sha1 passphrase 7 4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
            "timers pacing flood 7",
            "exit",
            "router ospfv3 vrf vrf03",
            "fips restrictions",
            "log-adjacency-changes detail",
            "address-family ipv6",
            "area 0.0.0.43 nssa no-summary",
            "area 0.0.0.43 range 20.1.1.0/24 not-advertise",
            "default-information originate route-map DefaultRouteFilter",
            "graceful-restart",
            "exit",
            "exit",
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(
            sorted(result["rendered"]), sorted(commands), result["rendered"]
        )

    def test_eos_ospfv3_parsed(self):
        commands = [
            "router ospfv3",
            "area 0.0.0.20 authentication ipsec spi 33 sha1 passphrase 7 4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
            "timers pacing flood 7",
            "exit",
            "router ospfv3 vrf vrf03",
            "fips restrictions",
            "log-adjacency-changes detail",
            "address-family ipv6",
            "area 0.0.0.43 nssa no-summary",
            "area 0.0.0.43 range 20.1.1.0/24 not-advertise",
            "default-information originate route-map DefaultRouteFilter",
            "graceful-restart",
            "exit",
            "exit",
        ]

        parsed_str = "\n".join(commands)
        set_module_args(dict(running_config=parsed_str, state="parsed"))
        result = self.execute_module(changed=False)
        parsed_list = {
            "processes": [
                {
                    "vrf": "default",
                    "areas": [
                        {
                            "area_id": "0.0.0.20",
                            "authentication": {
                                "algorithm": "sha1",
                                "spi": 33,
                                "hidden_key": True,
                                "passphrase": "4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
                            },
                        }
                    ],
                    "timers": {"pacing": 7},
                },
                {
                    "vrf": "vrf03",
                    "log_adjacency_changes": {"detail": True},
                    "fips_restrictions": True,
                    "address_family": [
                        {
                            "afi": "ipv6",
                            "areas": [
                                {
                                    "area_id": "0.0.0.43",
                                    "nssa": {"no_summary": True},
                                    "ranges": [
                                        {
                                            "address": "20.1.1.0/24",
                                            "advertise": False,
                                        }
                                    ],
                                }
                            ],
                            "default_information": {
                                "originate": True,
                                "route_map": "DefaultRouteFilter",
                            },
                            "graceful_restart": {"set": True},
                        }
                    ],
                },
            ]
        }
        self.assertEqual(sorted(parsed_list), sorted(result["parsed"]))

    def test_eos_ospfv3_gathered(self):
        set_module_args(dict(state="gathered"))
        result = self.execute_module(
            changed=False, filename="eos_ospfv3_config_gather.cfg"
        )
        gather_list = {
            "processes": [
                {
                    "vrf": "default",
                    "areas": [
                        {
                            "area_id": "0.0.0.20",
                            "authentication": {
                                "algorithm": "sha1",
                                "spi": 33,
                                "hidden_key": True,
                                "passphrase": "4O8T3zo4xBdRWXBnsnK934o9SEb+jEhHUN6+xzZgCo2j9EnQBUvtwNxxLEmYmm6w",
                            },
                        }
                    ],
                    "timers": {"pacing": 7},
                },
                {
                    "vrf": "vrf03",
                    "log_adjacency_changes": {"detail": True},
                    "fips_restrictions": True,
                    "address_family": [
                        {
                            "afi": "ipv6",
                            "areas": [
                                {
                                    "area_id": "0.0.0.43",
                                    "nssa": {"no_summary": True},
                                    "ranges": [
                                        {
                                            "address": "20.1.1.0/24",
                                            "advertise": False,
                                        }
                                    ],
                                }
                            ],
                            "default_information": {
                                "originate": True,
                                "route_map": "DefaultRouteFilter",
                            },
                            "graceful_restart": {"set": True},
                        }
                    ],
                },
            ]
        }
        self.assertEqual(sorted(gather_list), sorted(result["gathered"]))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


DOCUMENTATION = '''
---
module: ce_multicast_igmp_enable
version_added: '0.2.0'
author: xuxiaowei0512 (@CloudEngine-Ansible)
short_description: Manages multicast igmp enable configuration on HUAWEI CloudEngine switches.
description:
  - Manages multicast igmp on HUAWEI CloudEngine switches.
notes:
  - If no vrf is supplied, vrf is set to default.
    If I(state=absent), the route will be removed, regardless of the
    non-required parameters.
  - This module requires the netconf system service be enabled on
    the remote device being managed.
  - This module works with connection C(netconf).
options:
  aftype:
    description:
      - Destination ip address family type of static route.
    required: true
    type: str
    choices: ['v4','v6']
  features:
    description:
      - Distinguish between Globally Enabled IGMP or
      - Enabled IGMP under vlanID.
    required: true
    type: str
    choices: ['global','vlan']
  vlan_id:
    description:
      - Virtual LAN identity.
    type: int
  igmp:
    description:
      - Enable Layer 2 multicast Snooping in a VLAN.
    type: bool
    default: false
  version:
    description:
      - Specifies the IGMP version that can be processed.
    default: 2
    type: int
  proxy:
    description:
      - Layer 2 multicast snooping proxy is enabled.
    type: bool
    default: false
  state:
    description:
      - Specify desired state of the resource.
    choices: ['present','absent']
    default: present
    type: str
'''

EXAMPLES = '''

  - name: Configure global igmp enable
    community.network.ce_multicast_igmp_enable:
      aftype: v4
      features: 'global'
      state: present

  - name: Configure global igmp disable
    community.network.ce_multicast_igmp_enable:
      features: 'global'
      aftype: v4
      state: absent

  - name: Configure vlan igmp enable
    community.network.ce_multicast_igmp_enable:
      features: 'vlan'
      aftype: v4
      vlan_id: 1
      igmp: true

  - name: New proxy,igmp,version
    community.network.ce_multicast_igmp_enable:
      features: 'vlan'
      aftype: v4
      vlan_id: 1
      proxy: true
      igmp: true
      version: 1

  - name: Modify proxy,igmp,version
    community.network.ce_multicast_igmp_enable:
      features: 'vlan'
      aftype: v4
      vlan_id: 1
      version: 2

  - name: Delete proxy,igmp,version
    community.network.ce_multicast_igmp_enable:
      features: 'vlan'
      aftype: v4
      vlan_id: 1
      state: absent
'''
RETURN = '''
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"addrFamily": "ipv4unicast", "features": "vlan", "proxyEnable": "false",
            "snoopingEnable": "false",  "state": "absent", "version": 2, "vlanId": 1}
existing:
    description: k/v pairs of existing switchport
    returned: always
    type: dict
    sample: {}
end_state:
    description: k/v pairs of switchport after module execution
    returned: always
    type: dict
    sample: {}
updates:
    description: command list sent to the device
    returned: always
    type: list
    sample: ["undo igmp snooping enable",
        "undo igmp snooping version",
        "undo igmp snooping proxy"]
changed:
    description: check if a change was made on the device
    returned: always
    type: bool
    sample: true
'''

from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config

CE_NC_GET_IGMP_GLOBAL = """
<filter type="subtree">
  <l2mc xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <l2McSnpgEnables>
      <l2McSnpgEnable>
        <addrFamily>%s</addrFamily>
      </l2McSnpgEnable>
    </l2McSnpgEnables>
  </l2mc>
</filter>
"""
CE_NC_MERGE_IGMP_SYSVIEW = """
<l2mc xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <l2McSnpgEnables>
      <l2McSnpgEnable operation="merge">
        <addrFamily>%s</addrFamily>
      </l2McSnpgEnable>
    </l2McSnpgEnables>
</l2mc>
"""
CE_NC_DELETE_IGMP_SYSVIEW = """
<l2mc xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <l2McSnpgEnables>
      <l2McSnpgEnable operation="delete">
        <addrFamily>%s</addrFamily>
      </l2McSnpgEnable>
    </l2McSnpgEnables>
</l2mc>
"""
CE_NC_GET_IGMP_VLAN_INFO = """
<filter type="subtree">
  <l2mc xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <vlan>
      <l2McVlanCfgs>
        <l2McVlanCfg>
          <addrFamily>%s</addrFamily>
          <vlanId>%s</vlanId>
          <snoopingEnable></snoopingEnable>
          <version></version>
          <proxyEnable></proxyEnable>
        </l2McVlanCfg>
      </l2McVlanCfgs>
    </vlan>
  </l2mc>
</filter>
"""
CE_NC_MERGE_IGMP_VLANVIEW = """
<l2mc xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <vlan>
      <l2McVlanCfgs>
        <l2McVlanCfg operation="merge">
          <addrFamily>%s</addrFamily>
          <vlanId>%s</vlanId>%s%s%s
        </l2McVlanCfg>
      </l2McVlanCfgs>
    </vlan>
</l2mc>
"""
CE_NC_MERGE_IGMP_VLANVIEW_SNOENABLE = """
<snoopingEnable>%s</snoopingEnable>
"""
CE_NC_MERGE_IGMP_VLANVIEW_VERSION = """
<version>%s</version>
"""
CE_NC_MERGE_IGMP_VLANVIEW_PROXYENABLE = """
<proxyEnable>%s</proxyEnable>
"""
CE_NC_DELETE_IGMP_VLANVIEW = """
<l2mc xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <vlan>
      <l2McVlanCfgs>
        <l2McVlanCfg operation="delete">
          <addrFamily>%s</addrFamily>
          <vlanId>%s</vlanId>
        </l2McVlanCfg>
      </l2McVlanCfgs>
    </vlan>
</l2mc>
"""


def get_xml(xml, value):
    """operate xml"""
    tempxml = xml % value
    return tempxml


def build_config_xml(xmlstr):
    """build config xml"""

    return '<config> ' + xmlstr + ' </config>'


class IgmpSnoop(object):
    """igmp snooping module"""

    def __init__(self, argument_spec):
        """igmp snooping info"""
        self.spec = argument_spec
        self.module = None
        self._initmodule_()

        self.aftype = self.module.params['aftype']
        self.state = self.module.params['state']
        if self.aftype == "v4":
            self.addr_family = "ipv4unicast"
        else:
            self.addr_family = "ipv6unicast"
        self.features = self.module.params['features']
        self.vlan_id = self.module.params['vlan_id']
        self.igmp = str(self.module.params['igmp']).lower()
        self.version = self.module.params['version']
        if self.version is None:
            self.version = 2
        self.proxy = str(self.module.params['proxy']).lower()
        # state
        self.changed = False
        self.updates_cmd = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()
        self.igmp_info_data = dict()

    def _initmodule_(self):
        """init module"""
        self.module = AnsibleModule(
            argument_spec=self.spec, supports_check_mode=False)

    def _checkresponse_(self, xml_str, xml_name):
        """check if response message is already succeed."""

        if "<ok/>" not in xml_str:
            self.module.fail_json(msg='Error: %s failed.' % xml_name)

    def _checkparams_(self):
        """check all input params"""
        # check vlan id
        if self.features == 'vlan':
            if not self.vlan_id:
                self.module.fail_json(msg='Error: missing required arguments: vlan_id.')

        if self.vlan_id:
            if self.vlan_id <= 0 or self.vlan_id > 4094:
                self.module.fail_json(
                    msg='Error: Vlan id is not in the range from 1 to 4094.')
        # check  version
        if self.version:
            if self.version <= 0 or self.version > 3:
                self.module.fail_json(
                    msg='Error: Version id is not in the range from 1 to 3.')

    def set_change_state(self):
        """set change state"""
        state = self.state
        change = False
        # vlan view igmp
        if self.features == 'vlan':
            self.get_igmp_vlan()
            change = self.compare_data()
        else:
            # sys view igmp(global)
            self.get_igmp_global()
            # new or edit
            if state == 'present':
                if not self.igmp_info_data["igmp_info"]:
                    # igmp_info_data has not igmp_info value.
                    change = True
            else:
                # delete
                if self.igmp_info_data["igmp_info"]:
                    # igmp_info_data has not igmp_info value.
                    change = True
        self.changed = change

    def compare_data(self):
        """compare new data and old data"""
        state = self.state
        change = False
        # new or edit
        if state == 'present':
            # edit
            if self.igmp_info_data["igmp_info"]:
                for data in self.igmp_info_data["igmp_info"]:
                    if self.addr_family == data["addrFamily"] and str(self.vlan_id) == data["vlanId"]:
                        if self.igmp:
                            if self.igmp != data["snoopingEnable"]:
                                change = True
                        if self.version:
                            if str(self.version) != data["version"]:
                                change = True
                        if self.proxy:
                            if self.proxy != data["proxyEnable"]:
                                change = True
            # new
            else:
                change = True
        else:
            # delete
            if self.igmp_info_data["igmp_info"]:
                change = True
        return change

    def get_igmp_vlan(self):
        """get igmp vlan info data"""
        self.igmp_info_data["igmp_info"] = list()
        getxmlstr = CE_NC_GET_IGMP_VLAN_INFO % (self.addr_family, self.vlan_id)
        xml_str = get_nc_config(self.module, getxmlstr)
        if 'data/' in xml_str:
            return
        xml_str = xml_str.replace('\r', '').replace('\n', ''). \
            replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', ""). \
            replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
        root = ElementTree.fromstring(xml_str)
        igmp_enable = root.findall(
            "l2mc/vlan/l2McVlanCfgs/l2McVlanCfg")
        if igmp_enable:
            # igmp_enable = [{addressFamily:'xx'}]
            for igmp_enable_key in igmp_enable:
                # igmp_enable_key = {addressFamily:'xx'}
                igmp_global_info = dict()
                for ele in igmp_enable_key:
                    if ele.tag in ["addrFamily", "vlanId", "snoopingEnable", "version", "proxyEnable"]:
                        igmp_global_info[ele.tag] = ele.text
                self.igmp_info_data["igmp_info"].append(igmp_global_info)

    def get_igmp_global(self):
        """get igmp global data"""
        self.igmp_info_data["igmp_info"] = list()
        getxmlstr = CE_NC_GET_IGMP_GLOBAL % (
            self.addr_family)
        xml_str = get_nc_config(self.module, getxmlstr)
        if 'data/' in xml_str:
            return
        xml_str = xml_str.replace('\r', '').replace('\n', ''). \
            replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', ""). \
            replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
        root = ElementTree.fromstring(xml_str)
        igmp_enable = root.findall(
            'l2mc/l2McSnpgEnables/l2McSnpgEnable')
        if igmp_enable:
            # igmp_enable = [{addressFamily:'xx'}]
            for igmp_enable_key in igmp_enable:
                # igmp_enable_key = {addressFamily:'xx'}
                igmp_global_info = dict()
                for ele in igmp_enable_key:
                    if ele.tag in ["addrFamily"]:
                        igmp_global_info[ele.tag] = ele.text
                        self.igmp_info_data["igmp_info"].append(igmp_global_info)

    def set_vlanview_igmp(self):
        """set igmp of vlanview"""
        if not self.changed:
            return
        addr_family = self.addr_family
        state = self.state
        igmp_xml = """\n"""
        version_xml = """\n"""
        proxy_xml = """\n"""
        if state == "present":
            if self.igmp:
                igmp_xml = get_xml(CE_NC_MERGE_IGMP_VLANVIEW_SNOENABLE, self.igmp.lower())
            if str(self.version):
                version_xml = get_xml(CE_NC_MERGE_IGMP_VLANVIEW_VERSION, self.version)
            if self.proxy:
                proxy_xml = get_xml(CE_NC_MERGE_IGMP_VLANVIEW_PROXYENABLE, self.proxy.lower())
            configxmlstr = CE_NC_MERGE_IGMP_VLANVIEW % (
                addr_family, self.vlan_id, igmp_xml, version_xml, proxy_xml)
        else:
            configxmlstr = CE_NC_DELETE_IGMP_VLANVIEW % (addr_family, self.vlan_id)
        conf_str = build_config_xml(configxmlstr)
        recv_xml = set_nc_config(self.module, conf_str)
        self._checkresponse_(recv_xml, "SET_VLANVIEW_IGMP")

    def set_sysview_igmp(self):
        """set igmp of sysview"""
        if not self.changed:
            return
        version = self.addr_family
        state = self.state
        if state == "present":
            configxmlstr = CE_NC_MERGE_IGMP_SYSVIEW % (version)
        else:
            configxmlstr = CE_NC_DELETE_IGMP_SYSVIEW % (version)

        conf_str = build_config_xml(configxmlstr)
        recv_xml = set_nc_config(self.module, conf_str)
        self._checkresponse_(recv_xml, "SET_SYSVIEW_IGMP")

    def set_sysview_cmd(self):
        """set sysview update command"""
        if not self.changed:
            return
        if self.state == "present":
            self.updates_cmd.append('igmp snooping enable')
        else:
            self.updates_cmd.append('undo igmp snooping enable')

    def set_vlanview_cmd(self):
        """set vlanview update command"""
        if not self.changed:
            return
        if self.state == "present":
            if self.igmp:
                if self.igmp.lower() == 'true':
                    self.updates_cmd.append('igmp snooping enable')
                else:
                    self.updates_cmd.append('undo igmp snooping enable')
            if str(self.version):
                self.updates_cmd.append('igmp snooping version %s' % (self.version))
            else:
                self.updates_cmd.append('undo igmp snooping version')
            if self.proxy:
                if self.proxy.lower() == 'true':
                    self.updates_cmd.append('igmp snooping proxy')
                else:
                    self.updates_cmd.append('undo igmp snooping proxy')

        else:
            self.updates_cmd.append('undo igmp snooping enable')
            self.updates_cmd.append('undo igmp snooping version')
            self.updates_cmd.append('undo igmp snooping proxy')

    def get_existing(self):
        """get existing information"""
        self.set_change_state()
        self.existing["igmp_info"] = self.igmp_info_data["igmp_info"]

    def get_proposed(self):
        """get proposed information"""
        self.proposed['addrFamily'] = self.addr_family
        self.proposed['features'] = self.features
        if self.features == 'vlan':
            self.proposed['snoopingEnable'] = self.igmp
            self.proposed['version'] = self.version
            self.proposed['vlanId'] = self.vlan_id
            self.proposed['proxyEnable'] = self.proxy
        self.proposed['state'] = self.state

    def set_igmp_netconf(self):
        """config netconf"""
        if self.features == 'vlan':
            self.set_vlanview_igmp()
        else:
            self.set_sysview_igmp()

    def set_update_cmd(self):
        """set update command"""
        if self.features == 'vlan':
            self.set_vlanview_cmd()
        else:
            self.set_sysview_cmd()

    def get_end_state(self):
        """get end state information"""
        if self.features == 'vlan':
            self.get_igmp_vlan()
        else:
            # sys view igmp(global)
            self.get_igmp_global()
        self.end_state["igmp_info"] = self.igmp_info_data["igmp_info"]

    def work(self):
        """worker"""
        self._checkparams_()
        self.get_existing()
        self.get_proposed()
        self.set_igmp_netconf()
        self.set_update_cmd()
        self.get_end_state()
        self.results['changed'] = self.changed
        self.results['existing'] = self.existing
        self.results['proposed'] = self.proposed
        self.results['end_state'] = self.end_state
        if self.changed:
            self.results['updates'] = self.updates_cmd
        else:
            self.results['updates'] = list()
        self.module.exit_json(**self.results)


def main():
    """main"""
    argument_spec = dict(
        aftype=dict(choices=['v4', 'v6'], required=True),
        features=dict(required=True, choices=['global', 'vlan'], type='str'),
        vlan_id=dict(type='int'),
        igmp=dict(type='bool', default=False),
        version=dict(type='int', default=2),
        proxy=dict(type='bool', default=False),
        state=dict(choices=['absent', 'present'], default='present'),
    )
    interface = IgmpSnoop(argument_spec)
    interface.work()


if __name__ == '__main__':
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# (c) 2016 Red Hat Inc.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
from __future__ import absolute_import, division, print_function
__metaclass__ = type

import json
from ansible.module_utils._text import to_text
from ansible.module_utils.basic import env_fallback
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list, ComplexList
from ansible.module_utils.common._collections_compat import Mapping
from ansible.module_utils.connection import Connection, ConnectionError

_DEVICE_CONNECTION = None


class Cli:
    def __init__(self, module):
        self._module = module
        self._device_configs = {}
        self._connection = None

    def get_capabilities(self):
        """Returns platform info of the remove device
        """
        connection = self._get_connection()
        return json.loads(connection.get_capabilities())

    def _get_connection(self):
        if not self._connection:
            self._connection = Connection(self._module._socket_path)
        return self._connection

    def get_config(self, flags=None):
        """Retrieves the current config from the device or cache
        """
        flags = [] if flags is None else flags
        if self._device_configs == {}:
            connection = self._get_connection()
            try:
                out = connection.get_config(flags=flags)
            except ConnectionError as exc:
                self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))
            self._device_configs = to_text(out, errors='surrogate_then_replace').strip()
        return self._device_configs

    def run_commands(self, commands, check_rc=True):
        """Runs list of commands on remote device and returns results
        """
        connection = self._get_connection()
        try:
            response = connection.run_commands(commands=commands, check_rc=check_rc)
        except ConnectionError as exc:
            self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))
        return response

    def get_diff(self, candidate=None, running=None, diff_match='line', diff_ignore_lines=None, path=None, diff_replace='line'):
        conn = self._get_connection()
        try:
            diff = conn.get_diff(candidate=candidate, running=running, diff_match=diff_match,
                                 diff_ignore_lines=diff_ignore_lines, path=path, diff_replace=diff_replace)
        except ConnectionError as exc:
            self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))
        return diff


class HttpApi:
    def __init__(self, module):
        self._module = module
        self._device_configs = {}
        self._connection_obj = None

    def get_capabilities(self):
        """Returns platform info of the remove device
        """
        try:
            capabilities = self._connection.get_capabilities()
        except ConnectionError as exc:
            self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))

        return json.loads(capabilities)

    @property
    def _connection(self):
        if not self._connection_obj:
            self._connection_obj = Connection(self._module._socket_path)
        return self._connection_obj

    def get_config(self, flags=None):
        """Retrieves the current config from the device or cache
        """
        flags = [] if flags is None else flags
        if self._device_configs == {}:
            try:
                out = self._connection.get_config(flags=flags)
            except ConnectionError as exc:
                self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))
            self._device_configs = to_text(out, errors='surrogate_then_replace').strip()
        return self._device_configs

    def run_commands(self, commands, check_rc=True):
        """Runs list of commands on remote device and returns results
        """
        try:
            response = self._connection.run_commands(commands=commands, check_rc=check_rc)
        except ConnectionError as exc:
            self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))
        return response

    def send_requests(self, requests):
        """Send a list of http requests to remote device and return results
        """
        if requests is None:
            raise ValueError("'requests' value is required")

        responses = list()
        for req in to_list(requests):
            try:
                response = self._connection.send_request(**req)
            except ConnectionError as exc:
                self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))
            responses.append(response)
        return responses

    def get_diff(self, candidate=None, running=None, diff_match='line', diff_ignore_lines=None, path=None, diff_replace='line'):
        try:
            diff = self._connection.get_diff(candidate=candidate, running=running, diff_match=diff_match,
                                             diff_ignore_lines=diff_ignore_lines, path=path, diff_replace=diff_replace)
        except ConnectionError as exc:
            self._module.fail_json(msg=to_text(exc, errors='surrogate_then_replace'))
        return diff


def get_capabilities(module):
    conn = get_connection(module)
    return conn.get_capabilities()


def get_connection(module):
    global _DEVICE_CONNECTION
    if not _DEVICE_CONNECTION:
        connection_proxy = Connection(module._socket_path)
        cap = json.loads(connection_proxy.get_capabilities())
        if cap['network_api'] == 'cliconf':
            conn = Cli(module)
        elif cap['network_api'] == 'exosapi':
            conn = HttpApi(module)
        else:
            module.fail_json(msg='Invalid connection type %s' % cap['network_api'])
        _DEVICE_CONNECTION = conn
    return _DEVICE_CONNECTION


def get_config(module, flags=None):
    flags = None if flags is None else flags
    conn = get_connection(module)
    return conn.get_config(flags)


def load_config(module, commands):
    conn = get_connection(module)
    return conn.run_commands(to_command(module, commands))


def run_commands(module, commands, check_rc=True):
    conn = get_connection(module)
    return conn.run_commands(to_command(module, commands), check_rc=check_rc)


def to_command(module, commands):
    transform = ComplexList(dict(
        command=dict(key=True),
        output=dict(default='text'),
        prompt=dict(type='list'),
        answer=dict(type='list'),
        sendonly=dict(type='bool', default=False),
        check_all=dict(type='bool', default=False),
    ), module)
    return transform(to_list(commands))


def send_requests(module, requests):
    conn = get_connection(module)
    return conn.send_requests(to_request(module, requests))


def to_request(module, requests):
    transform = ComplexList(dict(
        path=dict(key=True),
        method=dict(),
        data=dict(type='dict'),
    ), module)
    return transform(to_list(requests))


def get_diff(module, candidate=None, running=None, diff_match='line', diff_ignore_lines=None, path=None, diff_replace='line'):
    conn = get_connection(module)
    return conn.get_diff(candidate=candidate, running=running, diff_match=diff_match, diff_ignore_lines=diff_ignore_lines, path=path, diff_replace=diff_replace)



# -*- coding: utf-8 -*-
# (c) 2018, Jordan Borean <jborean@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import pytest

from ansible_collections.ansible.windows.tests.unit.compat.mock import patch, MagicMock
from ansible_collections.ansible.windows.plugins.action.win_updates import ActionModule
from ansible.plugins.become.runas import BecomeModule
from ansible.playbook.task import Task


@pytest.fixture()
def test_win_updates():
    task = MagicMock(Task)
    task.args = {}

    connection = MagicMock()
    connection.module_implementation_preferences = ('.ps1', '.exe', '')

    play_context = MagicMock()
    play_context.check_mode = False

    plugin = ActionModule(task, connection, play_context, loader=None, templar=None, shared_loader_obj=None)
    return plugin


class TestWinUpdatesActionPlugin(object):

    INVALID_OPTIONS = (
        (
            {"state": "invalid"},
            False,
            "state must be either installed, searched or downloaded"
        ),
        (
            {"reboot": "nonsense"},
            False,
            "cannot parse reboot as a boolean: The value 'nonsense' is not a "
            "valid boolean."
        ),
        (
            {"reboot_timeout": "string"},
            False,
            "reboot_timeout must be an integer"
        ),
        (
            {"reboot": True},
            True,
            "async is not supported for this task when reboot=yes"
        )
    )

    # pylint bug: https://github.com/PyCQA/pylint/issues/511
    # pylint: disable=undefined-variable
    @pytest.mark.parametrize('task_args, async_val, expected',
                             ((t, a, e) for t, a, e in INVALID_OPTIONS))
    def test_invalid_options(self, task_args, async_val, expected):
        task = MagicMock(Task)
        task.args = task_args
        task.async_val = async_val

        connection = MagicMock()
        play_context = MagicMock()
        play_context.check_mode = False

        plugin = ActionModule(task, connection, play_context, loader=None,
                              templar=None, shared_loader_obj=None)
        res = plugin.run()
        assert res['failed']
        assert expected in res['msg']

    def test_exec_with_become(self, test_win_updates):
        test_become = os.urandom(8)

        set_become_mock = MagicMock()
        test_win_updates._connection.become = test_become
        test_win_updates._connection.set_become_plugin = set_become_mock

        with patch('ansible.plugins.action.ActionBase._execute_module', new=MagicMock()):
            test_win_updates._execute_module_with_become('win_updates', {}, {}, True, False)

        # Asserts we don't override the become plugin.
        assert set_become_mock.call_count == 1
        assert set_become_mock.mock_calls[0][1][0] == test_become

    def test_exec_with_become_no_plugin_set(self, test_win_updates):
        set_become_mock = MagicMock()
        test_win_updates._connection.become = None
        test_win_updates._connection.set_become_plugin = set_become_mock

        with patch('ansible.plugins.action.ActionBase._execute_module', new=MagicMock()):
            test_win_updates._execute_module_with_become('win_updates', {}, {}, True, False)

        assert set_become_mock.call_count == 2
        assert isinstance(set_become_mock.mock_calls[0][1][0], BecomeModule)
        assert set_become_mock.mock_calls[0][1][0].name == 'runas'
        assert set_become_mock.mock_calls[0][1][0].get_option('become_user') == 'SYSTEM'
        assert set_become_mock.mock_calls[0][1][0].get_option('become_flags') == ''
        assert set_become_mock.mock_calls[0][1][0].get_option('become_pass') is None
        assert set_become_mock.mock_calls[1][1] == (None,)

    def test_exec_with_become_no_plugin_set_use_task(self, test_win_updates):
        set_become_mock = MagicMock()
        test_win_updates._connection.become = None
        test_win_updates._connection.set_become_plugin = set_become_mock

        with patch('ansible.plugins.action.ActionBase._execute_module', new=MagicMock()):
            test_win_updates._execute_module_with_become('win_updates', {}, {}, True, True)

        assert set_become_mock.call_count == 1
        assert set_become_mock.mock_calls[0][1][0] is None

    def test_module_exec_async_result(self, monkeypatch):
        return_val = {
            "ansible_async_watchdog_pid": 7584,
            "ansible_job_id": "545519115287.9620",
            "changed": True,
            "finished": 0,
            "results_file": r"C:\.ansible_async\545519115287.9620",
            "started": 1
        }
        mock_execute = MagicMock(return_value=return_val)
        monkeypatch.setattr(ActionModule, '_execute_module', mock_execute)

        task = MagicMock(Task)
        task.args = {}
        task.async_val = 10

        connection = MagicMock()
        connection.module_implementation_preferences = ('.ps1', '.exe', '')

        play_context = MagicMock()
        play_context.check_mode = False
        play_context.become = True
        play_context.become_method = 'runas'
        play_context.become_user = 'SYSTEM'

        plugin = ActionModule(task, connection, play_context, loader=None,
                              templar=None, shared_loader_obj=None)
        actual = plugin.run(None, {})

        assert actual.get('failed') is None
        assert actual['ansible_async_watchdog_pid'] == 7584
        assert actual['ansible_job_id'] == "545519115287.9620"
        assert actual['changed'] is True
        assert actual['finished'] == 0
        assert actual['results_file'] == r"C:\.ansible_async\545519115287.9620"
        assert actual['started'] == 1



#!/usr/bin/env python
"""Verify the currently executing Shippable test matrix matches the one defined in the "shippable.yml" file."""
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import datetime
import json
import os
import re
import sys
import time

try:
    from typing import NoReturn
except ImportError:
    NoReturn = None

try:
    # noinspection PyCompatibility
    from urllib2 import urlopen  # pylint: disable=ansible-bad-import-from
except ImportError:
    # noinspection PyCompatibility
    from urllib.request import urlopen


def main():  # type: () -> None
    """Main entry point."""
    repo_full_name = os.environ['REPO_FULL_NAME']
    required_repo_full_name = 'ansible-collections/community.azure'

    if repo_full_name != required_repo_full_name:
        sys.stderr.write('Skipping matrix check on repo "%s" which is not "%s".\n' % (repo_full_name, required_repo_full_name))
        return

    with open('shippable.yml', 'rb') as yaml_file:
        yaml = yaml_file.read().decode('utf-8').splitlines()

    defined_matrix = [match.group(1) for match in [re.search(r'^ *- env: T=(.*)$', line) for line in yaml] if match and match.group(1) != 'none']

    if not defined_matrix:
        fail('No matrix entries found in the "shippable.yml" file.',
             'Did you modify the "shippable.yml" file?')

    run_id = os.environ['SHIPPABLE_BUILD_ID']
    sleep = 1
    jobs = []

    for attempts_remaining in range(4, -1, -1):
        try:
            jobs = json.loads(urlopen('https://api.shippable.com/jobs?runIds=%s' % run_id).read())

            if not isinstance(jobs, list):
                raise Exception('Shippable run %s data is not a list.' % run_id)

            break
        except Exception as ex:
            if not attempts_remaining:
                fail('Unable to retrieve Shippable run %s matrix.' % run_id,
                     str(ex))

            sys.stderr.write('Unable to retrieve Shippable run %s matrix: %s\n' % (run_id, ex))
            sys.stderr.write('Trying again in %d seconds...\n' % sleep)
            time.sleep(sleep)
            sleep *= 2

    if len(jobs) != len(defined_matrix):
        if len(jobs) == 1:
            hint = '\n\nMake sure you do not use the "Rebuild with SSH" option.'
        else:
            hint = ''

        fail('Shippable run %s has %d jobs instead of the expected %d jobs.' % (run_id, len(jobs), len(defined_matrix)),
             'Try re-running the entire matrix.%s' % hint)

    actual_matrix = dict((job.get('jobNumber'), dict(tuple(line.split('=', 1)) for line in job.get('env', [])).get('T', '')) for job in jobs)
    errors = [(job_number, test, actual_matrix.get(job_number)) for job_number, test in enumerate(defined_matrix, 1) if actual_matrix.get(job_number) != test]

    if len(errors):
        error_summary = '\n'.join('Job %s expected "%s" but found "%s" instead.' % (job_number, expected, actual) for job_number, expected, actual in errors)

        fail('Shippable run %s has a job matrix mismatch.' % run_id,
             'Try re-running the entire matrix.\n\n%s' % error_summary)


def fail(message, output):  # type: (str, str) -> NoReturn
    # Include a leading newline to improve readability on Shippable "Tests" tab.
    # Without this, the first line becomes indented.
    output = '\n' + output.strip()

    timestamp = datetime.datetime.utcnow().replace(microsecond=0).isoformat()

    # hack to avoid requiring junit-xml, which isn't pre-installed on Shippable outside our test containers
    xml = '''
<?xml version="1.0" encoding="utf-8"?>
<testsuites disabled="0" errors="1" failures="0" tests="1" time="0.0">
\t<testsuite disabled="0" errors="1" failures="0" file="None" log="None" name="ansible-test" skipped="0" tests="1" time="0" timestamp="%s" url="None">
\t\t<testcase classname="timeout" name="timeout">
\t\t\t<error message="%s" type="error">%s</error>
\t\t</testcase>
\t</testsuite>
</testsuites>
''' % (timestamp, message, output)

    path = 'shippable/testresults/check-matrix.xml'
    dir_path = os.path.dirname(path)

    if not os.path.exists(dir_path):
        os.makedirs(dir_path)

    with open(path, 'w') as junit_fd:
        junit_fd.write(xml.lstrip())

    sys.stderr.write(message + '\n')
    sys.stderr.write(output + '\n')

    sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright: (c) 2017 Ansible Project
# Copyright (c), Felix Fontein <felix@fontein.de>, 2019-2020
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import copy
import json
import pytest

from mock import MagicMock
from ansible_collections.community.hrobot.plugins.module_utils import robot


class ModuleFailException(Exception):
    def __init__(self, msg, **kwargs):
        super(ModuleFailException, self).__init__(msg)
        self.fail_msg = msg
        self.fail_kwargs = kwargs


def get_module_mock():
    def f(msg, **kwargs):
        raise ModuleFailException(msg, **kwargs)

    module = MagicMock()
    module.fail_json = f
    module.from_json = json.loads
    return module


# ########################################################################################

FETCH_URL_JSON_SUCCESS = [
    (
        (None, dict(
            body=json.dumps(dict(
                a='b'
            )).encode('utf-8'),
        )),
        None,
        (dict(
            a='b'
        ), None)
    ),
    (
        (None, dict(
            body=json.dumps(dict(
                error=dict(
                    code="foo",
                    status=400,
                    message="bar",
                ),
                a='b'
            )).encode('utf-8'),
        )),
        ['foo'],
        (dict(
            error=dict(
                code="foo",
                status=400,
                message="bar",
            ),
            a='b'
        ), 'foo')
    ),
]


FETCH_URL_JSON_FAIL = [
    (
        (None, dict(
            body=json.dumps(dict(
                error=dict(
                    code="foo",
                    status=400,
                    message="bar",
                ),
            )).encode('utf-8'),
        )),
        None,
        'Request failed: 400 foo (bar)'
    ),
    (
        (None, dict(
            body=json.dumps(dict(
                error=dict(
                    code="foo",
                    status=400,
                    message="bar",
                ),
            )).encode('utf-8'),
        )),
        ['bar'],
        'Request failed: 400 foo (bar)'
    ),
    (
        (None, dict(body='{this is not json}'.encode('utf-8'))),
        [],
        'Cannot decode content retrieved from https://foo/bar'
    ),
    (
        (None, dict()),
        [],
        'Cannot retrieve content from https://foo/bar'
    ),
]


@pytest.mark.parametrize("return_value, accept_errors, result", FETCH_URL_JSON_SUCCESS)
def test_fetch_url_json(monkeypatch, return_value, accept_errors, result):
    module = get_module_mock()
    robot.fetch_url = MagicMock(return_value=return_value)

    assert robot.fetch_url_json(module, 'https://foo/bar', accept_errors=accept_errors) == result


@pytest.mark.parametrize("return_value, accept_errors, result", FETCH_URL_JSON_FAIL)
def test_fetch_url_json_fail(monkeypatch, return_value, accept_errors, result):
    module = get_module_mock()
    robot.fetch_url = MagicMock(return_value=return_value)

    with pytest.raises(ModuleFailException) as exc:
        robot.fetch_url_json(module, 'https://foo/bar', accept_errors=accept_errors)

    assert exc.value.fail_msg == result
    assert exc.value.fail_kwargs == dict()


@pytest.mark.parametrize("return_value, accept_errors, result", FETCH_URL_JSON_SUCCESS)
def test_plugin_open_url_json(monkeypatch, return_value, accept_errors, result):
    response = MagicMock()
    response.read = MagicMock(return_value=return_value[1]['body'])
    robot.open_url = MagicMock(return_value=response)
    plugin = MagicMock()

    assert robot.plugin_open_url_json(plugin, 'https://foo/bar', accept_errors=accept_errors) == result


@pytest.mark.parametrize("return_value, accept_errors, result", FETCH_URL_JSON_FAIL)
def test_plugin_open_url_json_fail(monkeypatch, return_value, accept_errors, result):
    response = MagicMock()
    response.read = MagicMock(return_value=return_value[1].get('body', ''))
    robot.open_url = MagicMock(side_effect=robot.HTTPError('https://foo/bar', 400, 'Error!', {}, response))
    plugin = MagicMock()

    with pytest.raises(robot.PluginException) as exc:
        robot.plugin_open_url_json(plugin, 'https://foo/bar', accept_errors=accept_errors)

    assert exc.value.error_message == result


def test_plugin_open_url_json_fail_other(monkeypatch):
    robot.open_url = MagicMock(side_effect=Exception('buh!'))
    plugin = MagicMock()

    with pytest.raises(robot.PluginException) as exc:
        robot.plugin_open_url_json(plugin, 'https://foo/bar')

    assert exc.value.error_message == 'Failed request to Hetzner Robot server endpoint https://foo/bar: buh!'



#!/usr/bin/python
# Copyright: (c) 2017, Tim Rightnour <thegarbledone@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: syslogger
short_description: Log messages in the syslog
description:
    - Uses syslog to add log entries to the host.
options:
    msg:
        type: str
        description:
            - This is the message to place in syslog.
        required: True
    priority:
        type: str
        description:
            - Set the log priority.
        choices: [ "emerg", "alert", "crit", "err", "warning", "notice", "info", "debug" ]
        default: "info"
    facility:
        type: str
        description:
            - Set the log facility.
        choices: [ "kern", "user", "mail", "daemon", "auth", "lpr", "news",
                   "uucp", "cron", "syslog", "local0", "local1", "local2",
                   "local3", "local4", "local5", "local6", "local7" ]
        default: "daemon"
    log_pid:
        description:
            - Log the PID in brackets.
        type: bool
        default: False
    ident:
        description:
            - Specify the name of application name which is sending the log to syslog.
        type: str
        default: 'ansible_syslogger'
        version_added: '0.2.0'
author:
    - Tim Rightnour (@garbled1)
'''

EXAMPLES = r'''
- name: Simple Usage
  community.general.syslogger:
    msg: "I will end up as daemon.info"

- name: Send a log message with err priority and user facility with log_pid
  community.general.syslogger:
    msg: "Hello from Ansible"
    priority: "err"
    facility: "user"
    log_pid: true

- name: Specify the name of application which is sending log message
  community.general.syslogger:
    ident: "MyApp"
    msg: "I want to believe"
    priority: "alert"
'''

RETURN = r'''
ident:
  description: Name of application sending the message to log
  returned: always
  type: str
  sample: "ansible_syslogger"
  version_added: '0.2.0'
priority:
  description: Priority level
  returned: always
  type: str
  sample: "daemon"
facility:
  description: Syslog facility
  returned: always
  type: str
  sample: "info"
log_pid:
  description: Log PID status
  returned: always
  type: bool
  sample: True
msg:
  description: Message sent to syslog
  returned: always
  type: str
  sample: "Hello from Ansible"
'''

import syslog
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native


def get_facility(facility):
    return {
        'kern': syslog.LOG_KERN,
        'user': syslog.LOG_USER,
        'mail': syslog.LOG_MAIL,
        'daemon': syslog.LOG_DAEMON,
        'auth': syslog.LOG_AUTH,
        'lpr': syslog.LOG_LPR,
        'news': syslog.LOG_NEWS,
        'uucp': syslog.LOG_UUCP,
        'cron': syslog.LOG_CRON,
        'syslog': syslog.LOG_SYSLOG,
        'local0': syslog.LOG_LOCAL0,
        'local1': syslog.LOG_LOCAL1,
        'local2': syslog.LOG_LOCAL2,
        'local3': syslog.LOG_LOCAL3,
        'local4': syslog.LOG_LOCAL4,
        'local5': syslog.LOG_LOCAL5,
        'local6': syslog.LOG_LOCAL6,
        'local7': syslog.LOG_LOCAL7
    }.get(facility, syslog.LOG_DAEMON)


def get_priority(priority):
    return {
        'emerg': syslog.LOG_EMERG,
        'alert': syslog.LOG_ALERT,
        'crit': syslog.LOG_CRIT,
        'err': syslog.LOG_ERR,
        'warning': syslog.LOG_WARNING,
        'notice': syslog.LOG_NOTICE,
        'info': syslog.LOG_INFO,
        'debug': syslog.LOG_DEBUG
    }.get(priority, syslog.LOG_INFO)


def main():
    # define the available arguments/parameters that a user can pass to
    # the module
    module_args = dict(
        ident=dict(type='str', default='ansible_syslogger'),
        msg=dict(type='str', required=True),
        priority=dict(type='str', required=False,
                      choices=["emerg", "alert", "crit", "err", "warning",
                               "notice", "info", "debug"],
                      default='info'),
        facility=dict(type='str', required=False,
                      choices=["kern", "user", "mail", "daemon", "auth",
                               "lpr", "news", "uucp", "cron", "syslog",
                               "local0", "local1", "local2", "local3",
                               "local4", "local5", "local6", "local7"],
                      default='daemon'),
        log_pid=dict(type='bool', required=False, default=False)
    )

    module = AnsibleModule(
        argument_spec=module_args,
    )

    result = dict(
        changed=False,
        ident=module.params['ident'],
        priority=module.params['priority'],
        facility=module.params['facility'],
        log_pid=module.params['log_pid'],
        msg=module.params['msg']
    )

    # do the logging
    try:
        syslog.openlog(module.params['ident'],
                       syslog.LOG_PID if module.params['log_pid'] else 0,
                       get_facility(module.params['facility']))
        syslog.syslog(get_priority(module.params['priority']),
                      module.params['msg'])
        syslog.closelog()
        result['changed'] = True

    except Exception as exc:
        module.fail_json(error='Failed to write to syslog %s' % to_native(exc), exception=traceback.format_exc(), **result)

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# (c) 2018, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
---
module: scaleway_volume_facts
deprecated:
  removed_in: 3.0.0  # was Ansible 2.13
  why: Deprecated in favour of C(_info) module.
  alternative: Use M(community.general.scaleway_volume_info) instead.
short_description: Gather facts about the Scaleway volumes available.
description:
  - Gather facts about the Scaleway volumes available.
author:
  - "Yanis Guenane (@Spredzy)"
  - "Remy Leone (@sieben)"
extends_documentation_fragment:
- community.general.scaleway

options:
  region:
    type: str
    description:
     - Scaleway region to use (for example par1).
    required: true
    choices:
      - ams1
      - EMEA-NL-EVS
      - par1
      - EMEA-FR-PAR1
      - par2
      - EMEA-FR-PAR2
      - waw1
      - EMEA-PL-WAW1
'''

EXAMPLES = r'''
- name: Gather Scaleway volumes facts
  community.general.scaleway_volume_facts:
    region: par1
'''

RETURN = r'''
---
scaleway_volume_facts:
  description: Response from Scaleway API
  returned: success
  type: complex
  sample:
    "scaleway_volume_facts": [
        {
            "creation_date": "2018-08-14T20:56:24.949660+00:00",
            "export_uri": null,
            "id": "b8d51a06-daeb-4fef-9539-a8aea016c1ba",
            "modification_date": "2018-08-14T20:56:24.949660+00:00",
            "name": "test-volume",
            "organization": "3f709602-5e6c-4619-b80c-e841c89734af",
            "server": null,
            "size": 50000000000,
            "state": "available",
            "volume_type": "l_ssd"
        }
    ]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.scaleway import (
    Scaleway, ScalewayException, scaleway_argument_spec,
    SCALEWAY_LOCATION)


class ScalewayVolumeFacts(Scaleway):

    def __init__(self, module):
        super(ScalewayVolumeFacts, self).__init__(module)
        self.name = 'volumes'

        region = module.params["region"]
        self.module.params['api_url'] = SCALEWAY_LOCATION[region]["api_endpoint"]


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(dict(
        region=dict(required=True, choices=list(SCALEWAY_LOCATION.keys())),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        module.exit_json(
            ansible_facts={'scaleway_volume_facts': ScalewayVolumeFacts(module).get_resources()}
        )
    except ScalewayException as exc:
        module.fail_json(msg=exc.message)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_devtestlab_info
short_description: Get Azure DevTest Lab facts
description:
    - Get facts of Azure DevTest Lab.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
    name:
        description:
            - The name of the lab.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list

extends_documentation_fragment:
- azure.azcollection.azure


author:
    - Zim Kalinowski (@zikalino)
'''

EXAMPLES = '''
  - name: List instances of DevTest Lab by resource group
    community.azure.azure_rm_devtestlab_info:
      resource_group: testrg

  - name: List instances of DevTest Lab in subscription
    community.azure.azure_rm_devtestlab_info:

  - name: Get instance of DevTest Lab
    community.azure.azure_rm_devtestlab_info:
      resource_group: testrg
      name: testlab
'''

RETURN = '''
labs:
    description:
        - A list of dictionaries containing facts for Lab.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The identifier of the resource.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup/providers/microsoft.devtestlab/labs/myLab
        resource_group:
            description:
                - The name of the resource.
            returned: always
            type: str
            sample: testrg
        name:
            description:
                - The name of the resource.
            returned: always
            type: str
            sample: testlab
        location:
            description:
                - The location of the resource.
            returned: always
            type: str
            sample: eastus
        storage_type:
            description:
                - Lab storage type.
            returned: always
            type: str
            sample: standard
        premium_data_disks:
            description:
                - Are premium data disks allowed.
            returned: always
            type: bool
            sample: false
        provisioning_state:
            description:
                - Lab provisioning state.
            returned: always
            type: str
            sample: Succeeded
        artifacts_storage_account:
            description:
                - Artifacts storage account ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/myLab6346
        default_premium_storage_account:
            description:
                - Default premium storage account ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/myLab6346
        default_storage_account:
            description:
                - Default storage account ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/myLab6346
        premium_data_disk_storage_account:
            description:
                - Default storage account ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/myLab6346
        vault_name:
            description:
                - Key vault ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.KeyVault/vaults/myLab6788
        tags:
            description:
                - The tags of the resource.
            returned: always
            type: complex
            sample: "{ 'MyTag': 'MyValue' }"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from msrestazure.azure_exceptions import CloudError
    from azure.mgmt.devtestlabs import DevTestLabsClient
    from msrest.serialization import Model
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMDevTestLabInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str'
            ),
            name=dict(
                type='str'
            ),
            tags=dict(
                type='list'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.mgmt_client = None
        self.resource_group = None
        self.name = None
        self.tags = None
        super(AzureRMDevTestLabInfo, self).__init__(self.module_arg_spec, supports_tags=False)

    def exec_module(self, **kwargs):
        is_old_facts = self.module._name == 'azure_rm_devtestlab_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_devtestlab_facts' module has been renamed to 'azure_rm_devtestlab_info'", version='2.13')

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])
        self.mgmt_client = self.get_mgmt_svc_client(DevTestLabsClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if self.resource_group is not None:
            if self.name is not None:
                self.results['labs'] = self.get()
            else:
                self.results['labs'] = self.list_by_resource_group()
        else:
            self.results['labs'] = self.list_by_subscription()
        return self.results

    def list_by_resource_group(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.labs.list_by_resource_group(resource_group_name=self.resource_group)
            self.log("Response : {0}".format(response))
        except CloudError as e:
            self.log('Could not get facts for Lab.')

        if response is not None:
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_response(item))

        return results

    def list_by_subscription(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.labs.list_by_subscription()
            self.log("Response : {0}".format(response))
        except CloudError as e:
            self.log('Could not get facts for Lab.')

        if response is not None:
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_response(item))

        return results

    def get(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.labs.get(resource_group_name=self.resource_group,
                                                 name=self.name)
            self.log("Response : {0}".format(response))
        except CloudError as e:
            self.log('Could not get facts for Lab.')

        if response and self.has_tags(response.tags, self.tags):
            results.append(self.format_response(response))

        return results

    def format_response(self, item):
        d = item.as_dict()
        d = {
            'id': d.get('id', None),
            'resource_group': self.resource_group,
            'name': d.get('name', None),
            'location': d.get('location', '').replace(' ', '').lower(),
            'storage_type': d.get('lab_storage_type', '').lower(),
            'premium_data_disks': d.get('premium_data_disks') == 'Enabled',
            'provisioning_state': d.get('provisioning_state'),
            'artifacts_storage_account': d.get('artifacts_storage_account'),
            'default_premium_storage_account': d.get('default_premium_storage_account'),
            'default_storage_account': d.get('default_storage_account'),
            'premium_data_disk_storage_account': d.get('premium_data_disk_storage_account'),
            'vault_name': d.get('vault_name'),
            'tags': d.get('tags', None)
        }
        return d


def main():
    AzureRMDevTestLabInfo()


if __name__ == '__main__':
    main()
